@@ -1,8 +1,9 @@
-__version__ = '5.8.1'
+__version__ = '7.18.1'
 
 from functools import wraps
 
 import django
+from django.core.exceptions import ImproperlyConfigured
 
 from iommi._db_compat import (
     register_factory,
@@ -14,6 +15,7 @@ from iommi.base import MISSING
 from iommi.edit_table import (
     EditColumn,
     EditTable,
+    register_edit_column_factory,
 )
 from iommi.form import (
     Field,
@@ -43,14 +45,14 @@ from iommi.query import (
 )
 from iommi.sort_after import LAST
 from iommi.style import (
-    register_style,
     Style,
+    register_style,
 )
 from iommi.table import (
     Column,
+    Table,
     register_cell_formatter,
     register_column_factory,
-    Table,
 )
 
 setup_db_compat()
@@ -58,38 +60,54 @@ setup_db_compat()
 
 def render_if_needed(request, response):
     if isinstance(response, Part):
-        try:
-            if not response._is_bound:
-                response = response.bind(request=request)
-            return response.render_to_response()
-        except Exception as e:
-            filename, lineno = response._instantiated_at_info
-            from iommi.synthetic_traceback import SyntheticException
-
-            fake = SyntheticException(
-                tb=[dict(filename=filename, f_lineno=lineno, function='<iommi declaration>', f_globals={}, f_locals={})]
-            )
-
-            raise e from fake
+        return render_part(request, response)
     else:
         return response
 
 
-def middleware(get_response):
-    from django.db import connections
+def render_part(request, part: Part):
+    try:
+        if not part._is_bound:
+            part = part.bind(request=request)
+        return part.render_to_response()
+    except Exception as e:
+        filename, lineno = part._instantiated_at_info
+        if filename is None:
+            raise
 
-    atomic_request_connections = [db for db in connections.all() if db.settings_dict['ATOMIC_REQUESTS']]
-    if any(atomic_request_connections):
-        raise TypeError(
-            'The iommi middleware is unable to retain atomic transactions. Disable ATOMIC_REQUEST for '
-            f'database connections ({", ".join(db.settings_dict["NAME"] for db in atomic_request_connections)}) '
-            f'or remove middleware and use the @iommi_render decorator on the views instead.'
+        from iommi.synthetic_traceback import SyntheticException
+
+        fake = SyntheticException(
+            tb=[dict(filename=filename, f_lineno=lineno, function='<iommi declaration>', f_globals={}, f_locals={})]
         )
 
-    def iommi_middleware(request):
-        return render_if_needed(request, get_response(request))
+        raise e from fake
 
-    return iommi_middleware
+
+# noinspection PyPep8Naming
+class middleware:
+    def __init__(self, get_response):
+        from django.db import connections
+
+        self.get_response = get_response
+        self.atomic_db_aliases = {db.alias for db in connections.all() if db.settings_dict['ATOMIC_REQUESTS']}
+
+    def process_view(self, request, view_func, view_args, view_kwargs):
+        request.iommi_not_atomic_for = getattr(view_func, '_non_atomic_requests', set())
+
+    def __call__(self, request):
+        response = self.get_response(request)
+
+        if isinstance(response, Part):
+            from django.db import transaction
+
+            render = render_part
+            for alias in self.atomic_db_aliases:
+                if alias not in request.iommi_not_atomic_for:
+                    render = transaction.atomic(using=alias)(render)
+            return render(request, response)
+
+        return response
 
 
 def iommi_render(view):
@@ -123,6 +141,7 @@ __all__ = [
     'LAST',
     'MISSING',
     'register_factory',
+    'register_edit_column_factory',
     'register_field_factory',
     'register_filter_factory',
     'register_column_factory',
@@ -136,3 +155,12 @@ __all__ = [
 
 if django.VERSION[:2] < (3, 2):
     default_app_config = 'iommi.apps.IommiConfig'
+
+
+try:
+    from django.conf import settings
+
+    if 'iommi' not in settings.INSTALLED_APPS:
+        raise Exception("You must add 'iommi' to INSTALLED_APPS")
+except ImproperlyConfigured:
+    pass
