@@ -1,11 +1,10 @@
-from django.db.models import ManyToManyField
-from iommi.shortcut import Shortcut
-
 from iommi.base import MISSING
+from iommi.shortcut import Shortcut
 
 
 def setup_db_compat():
     setup_db_compat_django()
+    setup_db_compat_iommi()
 
 
 def register_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING, **kwargs):
@@ -19,11 +18,6 @@ def register_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSI
 
 
 def setup_db_compat_django():
-    from iommi.form import register_field_factory
-    from iommi.query import register_filter_factory
-    from iommi.table import register_column_factory
-    from iommi.sort_after import LAST
-
     from django.db.models import (
         AutoField,
         BinaryField,
@@ -32,6 +26,7 @@ def setup_db_compat_django():
         DateField,
         DateTimeField,
         DecimalField,
+        DurationField,
         EmailField,
         FileField,
         FilePathField,
@@ -40,6 +35,7 @@ def setup_db_compat_django():
         GenericIPAddressField,
         ImageField,
         IntegerField,
+        JSONField,
         ManyToManyField,
         ManyToManyRel,
         ManyToOneRel,
@@ -49,6 +45,11 @@ def setup_db_compat_django():
         UUIDField,
     )
 
+    from iommi.form import register_field_factory
+    from iommi.query import register_filter_factory
+    from iommi.sort_after import LAST
+    from iommi.table import register_column_factory
+
     def _get_choices_from_model_choices(model_field):
         return [value for value, label in model_field.choices]
 
@@ -56,44 +57,65 @@ def setup_db_compat_django():
         label_by_value = dict(model_field.choices)
 
         def choice_display_name_formatter(choice, **_):
-            return label_by_value[choice]
+            return label_by_value.get(choice, choice)
 
         return choice_display_name_formatter
 
-    def char_field_field_factory(model_field, **_):
-        if model_field.choices:
-            return Shortcut(
-                call_target__attribute='choice',
-                choices=_get_choices_from_model_choices(model_field),
-                choice_display_name_formatter=_build_display_name_formatter(model_field),
-            )
+    def model_choice_support_field_factory(shortcut_name='text'):
+        def fn(model_field, **_):
+            if model_field.choices:
+                return Shortcut(
+                    call_target__attribute='choice',
+                    choices=_get_choices_from_model_choices(model_field),
+                    choice_display_name_formatter=_build_display_name_formatter(model_field),
+                )
 
-        return Shortcut(call_target__attribute='text')
+            return Shortcut(call_target__attribute=shortcut_name)
 
-    def char_field_filter_factory(model_field, **_):
-        if model_field.choices:
-            return Shortcut(
-                call_target__attribute='choice',
-                choices=_get_choices_from_model_choices(model_field),
-                field__choice_display_name_formatter=_build_display_name_formatter(model_field),
-            )
+        return fn
 
-        return Shortcut(call_target__attribute='text')
+    def model_choice_support_filter_factory(shortcut_name='text'):
+        def fn(model_field, **_):
+            if model_field.choices:
+                return Shortcut(
+                    call_target__attribute='choice',
+                    choices=_get_choices_from_model_choices(model_field),
+                    field__choice_display_name_formatter=_build_display_name_formatter(model_field),
+                )
 
-    def char_field_column_factory(model_field, **_):
-        if model_field.choices:
-            return Shortcut(
-                call_target__attribute='choice',
-                choices=_get_choices_from_model_choices(model_field),
-                filter__field__choice_display_name_formatter=_build_display_name_formatter(model_field),
-            )
+            return Shortcut(call_target__attribute=shortcut_name)
 
-        return Shortcut(call_target__attribute='text')
+        return fn
 
-    register_field_factory(CharField, factory=char_field_field_factory)
-    register_filter_factory(CharField, factory=char_field_filter_factory)
-    register_column_factory(CharField, factory=char_field_column_factory)
+    def model_choice_support_column_factory(shortcut_name='text'):
+        def fn(model_field, **_):
+            if model_field.choices:
+                formatter = _build_display_name_formatter(model_field)
+                return Shortcut(
+                    call_target__attribute='choice',
+                    choices=_get_choices_from_model_choices(model_field),
+                    filter__field__choice_display_name_formatter=formatter,
+                    cell__format=lambda value, **_: formatter(value),
+                )
 
+            return Shortcut(call_target__attribute=shortcut_name)
+
+        return fn
+
+    register_field_factory(CharField, factory=model_choice_support_field_factory())
+    register_filter_factory(CharField, factory=model_choice_support_filter_factory())
+    register_column_factory(CharField, factory=model_choice_support_column_factory())
+
+    register_field_factory(IntegerField, factory=model_choice_support_field_factory(shortcut_name='integer'))
+    register_filter_factory(IntegerField, factory=model_choice_support_filter_factory(shortcut_name='integer'))
+    register_column_factory(IntegerField, factory=model_choice_support_column_factory(shortcut_name='integer'))
+
+    try:
+        from django.contrib.postgres.search import SearchVectorField
+
+        register_factory(SearchVectorField, factory=None)
+    except ImportError:
+        pass
     register_factory(UUIDField, shortcut_name='text')
     register_factory(TimeField, shortcut_name='time')
     register_factory(EmailField, shortcut_name='email')
@@ -101,7 +123,6 @@ def setup_db_compat_django():
     register_factory(DateField, shortcut_name='date')
     register_factory(DateTimeField, shortcut_name='datetime')
     register_factory(FloatField, shortcut_name='float')
-    register_factory(IntegerField, shortcut_name='integer')
     register_factory(FileField, shortcut_name='file')
     register_factory(AutoField, shortcut_name='integer', include=False)
     register_factory(
@@ -121,13 +142,8 @@ def setup_db_compat_django():
     register_factory(GenericIPAddressField, shortcut_name='text')
     register_factory(FilePathField, shortcut_name='text')
     register_factory(BinaryField, factory=None)
-    try:
-        # This raises ImportError when the postgres driver isn't installed
-        from django.contrib.postgres.fields import JSONField
-
-        register_factory(JSONField, shortcut_name='text', include=False)
-    except ImportError:
-        pass
+    register_factory(JSONField, shortcut_name='text', include=False)
+    register_factory(DurationField, shortcut_name='duration')
 
     # Column specific
     register_column_factory(BooleanField, shortcut_name='boolean')
@@ -153,6 +169,13 @@ def setup_db_compat_django():
     register_field_factory(FileField, shortcut_name='file')
 
 
+def setup_db_compat_iommi():
+    from iommi.model_fields import SortOrderField
+    from iommi.edit_table import register_edit_column_factory
+
+    register_edit_column_factory(SortOrderField, shortcut_name='reorder_handle')
+
+
 def base_defaults_factory(model_field):
     from iommi.base import capitalize
 
@@ -165,7 +188,7 @@ def base_defaults_factory(model_field):
 
 # TODO: move to form.py! remember to take the tests with them
 def field_defaults_factory(model_field):
-    from django.db.models import BooleanField
+    from django.db.models import BooleanField, ManyToManyField
 
     r = base_defaults_factory(model_field)
 
