@@ -1,31 +1,26 @@
 import functools
+from traceback import print_stack
 from typing import Type
 from urllib.parse import urlencode
 
 from django.apps import apps as django_apps
 from django.conf import settings
 from django.contrib import (
-    auth,
     messages,
 )
-from django.contrib.auth.hashers import check_password
-from django.contrib.auth.password_validation import validate_password
-from django.core.exceptions import ValidationError
 from django.db.models import Model
 from django.http import (
     Http404,
     HttpResponseRedirect,
 )
-from django.shortcuts import resolve_url
 from django.urls import (
     path,
     reverse,
 )
-from django.utils.safestring import mark_safe
-from django.utils.translation import gettext
+from django.utils.translation import gettext_lazy
 
 from iommi import (
-    Field,
+    EditTable,
     Form,
     Fragment,
     html,
@@ -38,6 +33,7 @@ from iommi import (
 from iommi._web_compat import format_html
 from iommi.base import (
     build_as_view_wrapper,
+    capitalize,
     items,
     values,
 )
@@ -48,10 +44,15 @@ from iommi.declarative.namespace import (
     Namespace,
     setdefaults_path,
 )
-from iommi.declarative.with_meta import with_meta
 from iommi.refinable import Refinable
 from iommi.shortcut import with_defaults
 from iommi.struct import Struct
+from iommi.views import (
+    auth_views,
+    change_password,
+    login,
+    logout,
+)
 
 app_verbose_name_by_label = {
     config.label: config.verbose_name.replace('_', ' ') for config in values(django_apps.app_configs)
@@ -64,7 +65,6 @@ joined_app_name_and_model = {
 }
 
 
-@with_meta
 class Messages(Fragment):
     class Meta:
         tag = 'div'
@@ -115,18 +115,49 @@ def read_config(f):
     return read_config_wrapper
 
 
-@with_meta  # we need @with_meta again here to make sure this constructor gets all the meta arguments first
+
 class Admin(Page):
     class Meta:
-        iommi_style = 'bootstrap'
-        table_class = Table
+        table_class = EditTable
         form_class = Form
-        apps = EMPTY
-        parts = EMPTY
-        apps__auth_user__include = True
-        apps__auth_group__include = True
-        parts__messages = Messages()
-        parts__list_auth_user = dict(
+
+    model: Type[Model] = Refinable()
+    instance: Model = Refinable()
+    app_name: str = Refinable()
+    model_name: str = Refinable()
+    operation: str = Refinable()
+    table_class: Type[Table] = Refinable()
+    form_class: Type[Form] = Refinable()
+
+    # Global configuration on apps level
+    apps: Namespace = Refinable()
+
+    menu = Menu(
+        include=lambda request, **_: getattr(request, 'iommi_main_menu', None) is None,
+        sub_menu=dict(
+            root=MenuItem(
+                url=lambda admin, **_: reverse('iommi.Admin.all_models', current_app=admin.app_name),
+                display_name=gettext_lazy('iommi administration'),
+            ),
+            change_password=MenuItem(
+                url=lambda admin, **_: reverse(change_password, current_app=admin.app_name),
+                display_name=gettext_lazy('Change password'),
+            ),
+            logout=MenuItem(
+                url=lambda admin, **_: reverse(logout, current_app=admin.app_name),
+                display_name=gettext_lazy('Logout'),
+            ),
+        ),
+    )
+
+    @read_config
+    @with_defaults(
+        apps__auth_user__include=True,
+        parts__edit_auth_user__fields__password__write_to_instance=lambda instance, value, **_: instance.set_password(value),
+        parts__edit_auth_user__fields__password__read_from_instance=lambda **_: '',
+        apps__auth_group__include=True,
+        parts__messages=Messages(),
+        parts__list_auth_user=dict(
             auto__include=['username', 'email', 'first_name', 'last_name', 'is_staff', 'is_active', 'is_superuser'],
             columns=dict(
                 username__filter=dict(
@@ -149,39 +180,18 @@ class Admin(Page):
                 is_active__filter__include=True,
                 is_superuser__filter__include=True,
             ),
-        )
-
-    model: Type[Model] = Refinable()
-    instance: Model = Refinable()
-    app_name: str = Refinable()
-    model_name: str = Refinable()
-    operation: str = Refinable()
-    table_class: Type[Table] = Refinable()
-    form_class: Type[Form] = Refinable()
-
-    # Global configuration on apps level
-    apps: Namespace = Refinable()
-
-    menu = Menu(
-        sub_menu=dict(
-            root=MenuItem(
-                url=lambda admin, **_: reverse('iommi.Admin.all_models'), display_name=gettext('iommi administration')
-            ),
-            change_password=MenuItem(
-                url=lambda **_: reverse(Auth.change_password), display_name=gettext('Change password')
-            ),
-            logout=MenuItem(url=lambda **_: reverse(Auth.logout), display_name=gettext('Logout')),
         ),
     )
-
-    @read_config
-    def __init__(self, parts, apps, **kwargs):
+    @dispatch(
+        apps=EMPTY,
+        parts=EMPTY,
+    )
+    def __init__(self, apps, parts, **kwargs):
         # Validate apps params
         for k in apps.keys():
-            assert (
-                k in joined_app_name_and_model
-            ), f'{k} is not a valid app/model key.\n\n' f'Valid keys:\n    ' + '\n    '.join(
-                sorted(joined_app_name_and_model)
+            assert k in joined_app_name_and_model, (
+                f'{k} is not a valid app/model key.\n\n'
+                f'Valid keys:\n    ' + '\n    '.join(sorted(joined_app_name_and_model))
             )
         super(Admin, self).__init__(parts=parts, apps=apps, **kwargs)
 
@@ -204,11 +214,15 @@ class Admin(Page):
 
     def as_view(self):
         def admin_view(request, *args, **kwargs):
+            app_name = kwargs.pop('app_name', None)
+
             if not getattr(request, 'user', None) or not request.user.is_authenticated:
-                return HttpResponseRedirect(f'{reverse(Auth.login)}?{urlencode(dict(next=request.path))}')
+                return HttpResponseRedirect(
+                    f'{reverse(login, current_app=app_name)}?{urlencode(dict(next=request.path))}'
+                )
 
             final_page = self.refine_with_params(
-                app_name=kwargs.pop('app_name', None),
+                app_name=app_name,
                 model_name=kwargs.pop('model_name', None),
                 pk=kwargs.pop('pk', None),
             ).refine_done()
@@ -226,6 +240,7 @@ class Admin(Page):
 
     def on_refine_done(self):
         part_name = ''
+        assert self.operation
         part_name += self.operation
         if self.app_name:
             part_name += '_' + self.app_name
@@ -296,57 +311,64 @@ class Admin(Page):
         operation='all_models',
     )
     def all_models(cls, table=None, **kwargs):
-        def rows(admin, request, included_filter=False, **_):
+        def rows_raw(admin, request, included_filter=False, **_):
             for app_name, models in items(django_apps.all_models):
-                has_yielded_header = False
-
                 for model_name, model in sorted(items(models), key=lambda x: x[1]._meta.verbose_name_plural):
                     key = f'{app_name}_{model_name}'
-                    included = admin.apps.get(key, {}).get('include', False)
+                    conf = admin.apps.get(key, {})
+                    included = conf.get('include', False)
                     if included == included_filter:
                         continue
 
                     if not cls.has_permission(request, instance=None, model=model, operation='view'):
                         continue
 
-                    if not has_yielded_header:
-                        yield Struct(
-                            name=app_verbose_name_by_label[app_name],
-                            verbose_app_name=app_verbose_name_by_label[app_name],
-                            app_name=None,
-                            model_name=None,
-                            url=None,
-                            format=lambda row, table, **_: (
-                                html.h2(row.name, _name='invalid_name').bind(parent=table).__html__()
-                            ),
-                            key=None,
-                        )
-                        has_yielded_header = True
+                    group = conf.get('group', app_verbose_name_by_label[app_name])
 
                     yield Struct(
-                        verbose_app_name=app_verbose_name_by_label[app_name],
+                        name=conf.get('name', model._meta.verbose_name_plural.capitalize()),
+                        group=group,
                         app_name=app_name,
-                        model_name=app_name,
-                        name=model._meta.verbose_name_plural.capitalize(),
-                        url='%s/%s/' % (app_name, model_name),
+                        model_name=model_name,
+                        url='{}/{}/'.format(app_name, model_name),
                         format=lambda row, **_: row.name,
                         key=key,
                     )
 
+        def rows(admin, request, included_filter=False, **_):
+            last_group = None
+            for row in sorted(rows_raw(admin=admin, request=request, included_filter=included_filter), key=lambda row: (row.group, row.name)):
+                if last_group != row.group:
+                    yield Struct(
+                        name=row.group,
+                        url=None,
+                        format=lambda row, table, **_: (
+                            html.h1(row.name, _name='invalid_name').bind(parent=table).__html__()
+                        ),
+                        key=None,
+                    )
+                    last_group = row.group
+                yield row
+
         table = setdefaults_path(
             Namespace(),
             table if table is not None else {},
-            title=gettext('All models'),
+            title='',
             call_target__cls=cls.get_meta().table_class,
             call_target__attribute='div',
             sortable=False,
             rows=rows,
             page_size=None,
             columns__name=dict(
-                cell__url=lambda row, **_: row.url,
+                cell__url=lambda row, **_: row.url if getattr(row, 'pk', None) != '#sentinel#' else None,
                 display_name='',
-                cell__format=lambda row, **format_kwargs: row.format(row=row, **format_kwargs),
+                cell__format=lambda row, **format_kwargs: (
+                    row.format(row=row, **format_kwargs) if getattr(row, 'pk', None) != '#sentinel#' else ''
+                ),
             ),
+            edit_actions__add_row__include=False,
+            edit_actions__save__include=False,
+            attrs__class__table=False,
         )
 
         add_models = setdefaults_path(
@@ -354,8 +376,10 @@ class Admin(Page):
             include=settings.DEBUG,
             call_target__cls=cls.get_meta().table_class,
             sortable=False,
-            rows=functools.partial(rows, included_filter=True),
+            create_form=None,
+            rows=functools.partial(rows_raw, included_filter=True),
             page_size=None,
+            columns__app_name=cls.get_meta().table_class.get_meta().member_class(auto_rowspan=True),
             columns__conf=(
                 cls.get_meta()
                 .table_class.get_meta()
@@ -381,20 +405,24 @@ class Admin(Page):
             ),
             columns__name=dict(
                 display_name='',
-                cell__format=lambda row, **format_kwargs: row.format(row=row, **format_kwargs),
+                cell__format=lambda row, **format_kwargs: (
+                    row.format(row=row, **format_kwargs) if getattr(row, 'pk', None) != '#sentinel#' else ''
+                ),
             ),
+            edit_actions__add_row__include=False,
+            edit_actions__save__include=False,
         )
 
         return cls(
             parts=dict(
                 all_models=table,
-                add_models_title=html.h1(gettext('Add models to admin'), include=settings.DEBUG),
+                add_models_title=html.h1(gettext_lazy('Add models to admin'), include=settings.DEBUG),
                 add_models_help_text=html.p(
                     format_html(
                         '''
                             Copy the conf value to the `Meta` class of an `iommi_admin.py` file.
 
-                            Read <a href="https://docs.iommi.rocks/en/latest/admin.html#customization">the docs for admin customization</a> for more information.
+                            Read <a href="https://docs.iommi.rocks//admin.html#customization">the docs for admin customization</a> for more information.
                         ''',
                     ),
                     include=settings.DEBUG,
@@ -414,7 +442,6 @@ class Admin(Page):
             table if table is not None else {},
             call_target__cls=cls.get_meta().table_class,
             columns=dict(
-                select__include=True,
                 edit=dict(
                     call_target__attribute='edit',
                     after=0,
@@ -423,7 +450,6 @@ class Admin(Page):
                 delete=dict(
                     call_target__attribute='delete',
                     after=LAST,
-                    cell__url=lambda row, **_: '%s/delete/' % row.pk,
                     include=lambda request, table, **_: (
                         cls.has_permission(request, instance=None, model=table.model, operation='delete')
                     ),
@@ -432,15 +458,12 @@ class Admin(Page):
             actions=dict(
                 create=dict(
                     display_name=lambda page, **_: (
-                        gettext('Create %(model_name)s') % dict(model_name=page.model._meta.verbose_name)
+                        gettext_lazy('Create %(model_name)s') % dict(model_name=page.model._meta.verbose_name)
                     ),
                     attrs__href='create/',
                 ),
             ),
             query_from_indexes=True,
-            bulk__actions__delete__include=lambda request, table, **_: (
-                cls.has_permission(request, instance=None, model=table.model, operation='delete')
-            ),
         )
 
         return cls(
@@ -522,115 +545,17 @@ class Admin(Page):
                 path('', cls.all_models().as_view(), name='iommi.Admin.all_models'),
                 path('<app_name>/<model_name>/', cls.list().as_view(), name='iommi.Admin.list'),
                 path('<app_name>/<model_name>/create/', cls.create().as_view(), name='iommi.Admin.create'),
-                path('<app_name>/<model_name>/<int:pk>/edit/', cls.edit().as_view(), name='iommi.Admin.edit'),
-                path('<app_name>/<model_name>/<int:pk>/delete/', cls.delete().as_view(), name='iommi.Admin.delete'),
-            ]
-            + Auth.urls().urlpatterns
-        )
-
-
-class Auth:
-    @classmethod
-    def login(cls, request):
-        return LoginPage()
-
-    @classmethod
-    def logout(cls, request):
-        auth.logout(request)
-        return HttpResponseRedirect(resolve_url(settings.LOGOUT_REDIRECT_URL or '/'))
-
-    @classmethod
-    def change_password(cls, request):
-        return ChangePasswordPage()
-
-    @classmethod
-    def urls(cls):
-        return Struct(
-            urlpatterns=[
-                path('login/', cls.login),
-                path('logout/', cls.logout),
-                path('change_password/', cls.change_password),
+                path('<app_name>/<model_name>/<pk>/edit/', cls.edit().as_view(), name='iommi.Admin.edit'),
+                path('<app_name>/<model_name>/<pk>/delete/', cls.delete().as_view(), name='iommi.Admin.delete'),
+                path('', auth_views()),
             ]
         )
 
-
-class LoginForm(Form):
-    username = Field(display_name=gettext('Username'))
-    password = Field.password(display_name=gettext('Password'))
-
-    class Meta:
-        title = gettext('Login')
-
-        @staticmethod
-        def actions__submit__post_handler(form, **_):
-            if form.is_valid():
-                user = auth.authenticate(
-                    username=form.fields.username.value,
-                    password=form.fields.password.value,
-                )
-
-                if user is not None:
-                    request = form.get_request()
-                    auth.login(request, user)
-                    return HttpResponseRedirect(request.GET.get('next', '/'))
-
-                form.add_error(gettext('Unknown username or password'))
-
-
-class LoginPage(Page):
-    form = LoginForm()
-    set_focus = html.script(
-        mark_safe(
-            'document.getElementById("id_username").focus();',
+    @classmethod
+    def m(cls):
+        from iommi.experimental.main_menu import M
+        return M(
+            view=cls.all_models(),
+            display_name=capitalize(gettext_lazy('admin')),
+            paths=cls.urls().urlpatterns,
         )
-    )
-
-
-def current_password__is_valid(form, parsed_data, **_):
-    return (
-        (True, None)
-        if check_password(parsed_data, form.get_request().user.password)
-        else (False, gettext('Incorrect password'))
-    )
-
-
-def new_password__is_valid(form, parsed_data, **_):
-    try:
-        validate_password(parsed_data, form.get_request().user)
-        return True, None
-    except ValidationError as e:
-        return False, ','.join(e)
-
-
-def confirm_password__is_valid(form, parsed_data, **_):
-    return (
-        (True, None)
-        if parsed_data == form.fields.new_password.value
-        else (False, gettext('New passwords does not match'))
-    )
-
-
-class ChangePasswordForm(Form):
-    class Meta:
-        title = gettext('Change password')
-
-        @staticmethod
-        def actions__submit__post_handler(form, request, **_):
-            if form.is_valid():
-                user = request.user
-                user.set_password(form.fields.new_password.value)
-                user.save()
-                return HttpResponseRedirect('..')
-
-    current_password = Field.password(is_valid=current_password__is_valid, display_name=gettext('Current password'))
-    new_password = Field.password(is_valid=new_password__is_valid, display_name=gettext('New password'))
-    confirm_password = Field.password(is_valid=confirm_password__is_valid, display_name=gettext('Confirm password'))
-
-
-class ChangePasswordPage(Page):
-    form = ChangePasswordForm()
-    set_focus = html.script(
-        mark_safe(
-            'document.getElementById("id_current_password").focus();',
-        )
-    )
