@@ -1,4 +1,5 @@
 from iommi.fragment import Fragment
+from iommi.refinable import Refinable
 from iommi.shortcut import with_defaults
 
 
@@ -16,6 +17,11 @@ class Asset(Fragment):
     each other, resulting in only one asset with a given name being rendered.
     """
 
+    in_body: bool = Refinable()
+
+    class Meta:
+        in_body = False
+
     @classmethod
     @with_defaults(
         tag='script',
