@@ -1,15 +1,17 @@
-from iommi.declarative.namespace import Namespace
-
 from iommi._web_compat import mark_safe
 from iommi.base import items
+from iommi.declarative.namespace import (
+    flatten,
+    Namespace,
+)
 from iommi.evaluate import (
-    evaluate_as_needed,
+    evaluate_as_needed_recursively,
     evaluate_strict,
 )
+from iommi.debug import iommi_debug_on
 
 
 def evaluate_attrs(obj, **kwargs):
-    from iommi.debug import iommi_debug_on
 
     attrs = obj.attrs or {}
 
@@ -42,14 +44,48 @@ or
         _parent=obj,
         **{
             **{
-                'class': evaluate_as_needed(classes, kwargs),
-                'style': evaluate_as_needed(styles, kwargs),
+                'class': evaluate_as_needed_recursively(classes, kwargs),
+                'style': evaluate_as_needed_recursively(styles, kwargs),
             },
-            **evaluate_as_needed(attrs, kwargs, ignore=('class', 'style')),
+            **evaluate_as_needed_recursively(attrs, kwargs, ignore=('class', 'style')),
         },
     )
 
 
+def _render_attrs_parts(attrs):
+    for key, value in sorted(attrs.items()):
+        if value is None:
+            continue
+        if value is True:
+            yield f'{key}'
+            continue
+        if isinstance(value, dict):
+            if key == 'class':
+                if not value:
+                    continue
+                value = render_class(value)
+                if not value:
+                    continue
+            elif key == 'style':
+                if not value:
+                    continue
+                value = render_style(value)
+                if not value:
+                    continue
+            else:
+                raise TypeError(f'Only the class and style attributes can be dicts, you sent {value} for key {key}')
+        elif isinstance(value, (list, tuple)):
+            raise TypeError(f"Attributes can't be of type {type(value).__name__}, you sent {value} for key {key}")
+        elif callable(value):
+            from .docs import get_docs_callable_description
+
+            raise TypeError(
+                f"Attributes can't be callable, you sent {get_docs_callable_description(value)} for key {key}"
+            )
+        v = f'{value}'.replace('"', '&quot;')
+        yield f'{key}="{v}"'
+
+
 def render_attrs(attrs):
     """
     Render HTML attributes, or return '' if no attributes needs to be rendered.
@@ -57,40 +93,7 @@ def render_attrs(attrs):
     if not attrs:
         return ''
 
-    def parts():
-        for key, value in sorted(attrs.items()):
-            if value is None:
-                continue
-            if value is True:
-                yield f'{key}'
-                continue
-            if isinstance(value, dict):
-                if key == 'class':
-                    if not value:
-                        continue
-                    value = render_class(value)
-                    if not value:
-                        continue
-                elif key == 'style':
-                    if not value:
-                        continue
-                    value = render_style(value)
-                    if not value:
-                        continue
-                else:
-                    raise TypeError(f'Only the class and style attributes can be dicts, you sent {value} for key {key}')
-            elif isinstance(value, (list, tuple)):
-                raise TypeError(f"Attributes can't be of type {type(value).__name__}, you sent {value} for key {key}")
-            elif callable(value):
-                from .docs import get_docs_callable_description
-
-                raise TypeError(
-                    f"Attributes can't be callable, you sent {get_docs_callable_description(value)} for key {key}"
-                )
-            v = f'{value}'.replace('"', '&quot;')
-            yield f'{key}="{v}"'
-
-    r = mark_safe(' %s' % ' '.join(parts()))
+    r = mark_safe(f" {' '.join(_render_attrs_parts(attrs))}")
     return '' if r == ' ' else r
 
 
@@ -150,8 +153,6 @@ class Attrs(Namespace):
     """
 
     def __init__(self, _parent, **attrs):
-        from iommi.debug import iommi_debug_on
-
         if iommi_debug_on() and getattr(_parent, '_name', None) is not None:
             attrs['data-iommi-path'] = _parent.iommi_dunder_path
             attrs['data-iommi-type'] = type(_parent).__name__
@@ -167,8 +168,8 @@ class Attrs(Namespace):
 
 
 def render_class(class_dict):
-    return ' '.join(sorted(name for name, flag in items(class_dict) if flag))
+    return ' '.join(sorted(name for name, flag in items(flatten(class_dict)) if flag))
 
 
 def render_style(class_dict):
-    return '; '.join(sorted(f'{k}: {v}' for k, v in items(class_dict) if v))
+    return '; '.join(sorted(f'{k}: {v}' for k, v in items(flatten(class_dict)) if v))
