@@ -1,6 +1,7 @@
 from django.conf import settings
 from django.db.models import QuerySet
 from django.utils.encoding import force_str
+from django.utils.functional import keep_lazy_text
 from django.utils.safestring import SafeText
 
 NOT_BOUND_MESSAGE = (
@@ -14,7 +15,7 @@ class UnknownMissingValueException(Exception):
 
 class Missing:
     def __bool__(self):
-        raise UnknownMissingValueException('MISSING is neither True nor False, is is unknown')
+        raise UnknownMissingValueException('MISSING is neither True nor False, it is unknown')
 
     def __str__(self):
         return 'MISSING'
@@ -55,6 +56,7 @@ def build_as_view_wrapper(target):
     return view_wrapper
 
 
+@keep_lazy_text
 def capitalize(s):
     if isinstance(s, SafeText):
         return SafeText(capitalize('' + s))  # str(s) will give you back SafeText, and then we have infinite recursion
