@@ -6,24 +6,18 @@ from os.path import (
     join,
 )
 
-from iommi.struct import Struct
+from django.http import HttpResponse
+from django.template.loader import get_template
 
 from iommi._web_compat import (
-    format_html,
     HttpRequest,
-    mark_safe,
     Template,
+    format_html,
+    mark_safe,
 )
 from iommi.base import items
-from iommi.member import (
-    MemberBinder,
-    Members,
-)
-from iommi.traversable import (
-    declared_members,
-    PathNotFoundException,
-    Traversable,
-)
+from iommi.struct import Struct
+
 from ._web_compat import settings
 
 
@@ -48,6 +42,16 @@ def endpoint__debug_tree(endpoint, **_):
     assert root._is_bound
 
     def rows(node, name='', path=None):
+        from iommi.member import (
+            MemberBinder,
+            Members,
+        )
+        from iommi.traversable import (
+            PathNotFoundException,
+            Traversable,
+            declared_members,
+        )
+
         if path is None:
             path = []
         is_struct = type(node) is Struct
@@ -59,10 +63,11 @@ def endpoint__debug_tree(endpoint, **_):
             p = None
 
         type_name = type(node).__name__ if not is_struct else None
-        base_type_name = type_name
+        prefix = ''
         if isinstance(node, Members):
             if node._declared_members:
-                type_name = f'Members[{node._cls.__name__}]'
+                type_name = node._cls.__name__
+                prefix = 'Contains '
             else:
                 return
 
@@ -70,7 +75,6 @@ def endpoint__debug_tree(endpoint, **_):
         if isinstance(node, dict):
             children = list(node.items())
         elif isinstance(node, Traversable):
-
             children = []
             for k, v in items(declared_members(node)):
                 if node._bound_members is not None and isinstance(node._bound_members.get(k, None), MemberBinder):
@@ -80,11 +84,18 @@ def endpoint__debug_tree(endpoint, **_):
         if (isinstance(node, Members) or isinstance(node, dict)) and not children:
             return
 
+        display_name = type_name
+        shortcut_stack = getattr(node, 'iommi_shortcut_stack', None)
+        shortcut = shortcut_stack[0] if shortcut_stack else None
+        if shortcut:
+            display_name = f'{type_name}.{shortcut}'
         yield Struct(
             name=name,
             obj=node,
-            type=type_name,
-            base_type=base_type_name,
+            prefix=prefix,
+            type_name=type_name,
+            display_name=display_name,
+            shortcut=shortcut,
             path=p,
             dunder_path='__'.join(path),
             included=is_bound,
@@ -123,18 +134,30 @@ def endpoint__debug_tree(endpoint, **_):
             """
             )
             sortable = False
-            row__attrs__class__included = lambda row, **_: row.included
             page_size = None
 
+            @staticmethod
+            def row__attrs__class__included(row, **_):
+                return row.included
+
         dunder_path = Column(
             cell__value=lambda row, **_: row.dunder_path,
             cell__format=dunder_path__format,
         )
         path = Column()
         type = Column(
-            cell__url=lambda row, **_: f'https://docs.iommi.rocks/en/latest/{row.base_type}.html'
-            if row.base_type
-            else None
+            attr='type_name',
+            cell__format=lambda row, **_: (
+                format_html(
+                    '{prefix}<a href="https://docs.iommi.rocks//{type_name}.html{anchor}">{display_name}</a>',
+                    prefix=row.prefix,
+                    type_name=row.type_name,
+                    anchor=f'#{row.shortcut.replace("_", "-")}' if row.shortcut else '',
+                    display_name=row.display_name,
+                )
+                if row.type_name
+                else ''
+            ),
         )
         included = Column.boolean()
 
@@ -147,12 +170,57 @@ def endpoint__debug_tree(endpoint, **_):
     return TreeTable(rows=rows(root)).bind(request=request)
 
 
+def endpoint__debug_templates_used(endpoint, request, **_):
+    root = endpoint.iommi_root()
+    from iommi.part import render_root
+
+    render_root(part=root)
+
+    def url_for_template(t):
+        if isinstance(t, str):
+            t = get_template(t).origin.name
+        else:
+            t = t.origin.name
+
+        return local_debug_url_builder(t, 1)
+
+    unique_templates = list(dict.fromkeys(request.iommi_used_templates))
+    links = [
+        format_html(
+            '<li><a href="{}">{}</a></li>',
+            url_for_template(t),
+            t,
+        )
+        for t in unique_templates
+    ]
+    links = format_html('{}' * len(links), *links)
+    return HttpResponse(
+        # language=HTML
+        format_html(
+            '''
+                <html>
+                    <head>
+                        <style>
+                        </style>
+                    </head>
+                    <body>
+                        <ul>
+                            {}
+                        </ul>
+                    </body>
+                </html>
+            ''',
+            links,
+        )
+    )
+
+
 def local_debug_url_builder(filename, lineno):
     if not isabs(filename):
         filename = join(settings.BASE_DIR, filename)
     if hasattr(settings, 'IOMMI_DEBUG_URL_MAPPING'):
         filename = filename.replace(*settings.IOMMI_DEBUG_URL_MAPPING)
-    return "pycharm://open?file=%s" % (filename,) + ('' if lineno is None else "&line=%d" % (lineno,))
+    return f"pycharm://open?file={filename}" + ('' if lineno is None else f"&line={lineno:d}")
 
 
 def src_debug_url_builder(filename, lineno=None):
@@ -213,6 +281,7 @@ def get_instantiated_at_info(frame):
 
 
 def filename_and_line_num_from_part(part):
+    from iommi.traversable import Traversable
     filename, line_num = getattr(part, '_instantiated_at_info', (None, None))
     if (filename, line_num) == (None, None) and isinstance(part, Traversable):
         # inspect.findsource() parses the full AST of the file, so beware the performance implications
@@ -224,6 +293,7 @@ def filename_and_line_num_from_part(part):
 
 def iommi_debug_panel(part):
     source_url = source_url_from_part(part)
+    # language=js
     script = r"""
         window.iommi_start_pick = function() {
             window.iommi_pick_stack = [];
@@ -254,7 +324,7 @@ def iommi_debug_panel(part):
                 toolbar.append(create('<div style="float: right" onclick="iommi_close_pick_toolbar()">close</div>'));
                 for (let i in window.iommi_pick_stack) {
                     let x = window.iommi_pick_stack[i];
-                    toolbar.append(create('<div style="background-color: ' + getColor(i) + '">' + x[0] + ' <a href="https://docs.iommi.rocks/en/latest/' + x[1] + '.html">' + x[1] + '</a></div>'));
+                    toolbar.append(create('<div style="background-color: ' + getColor(i) + '">' + x[0] + ' <a href="https://docs.iommi.rocks//' + x[1] + '.html">' + x[1] + '</a></div>'));
                 }
             }
 
