@@ -1,7 +1,8 @@
 import functools
 
-from .namespace import Namespace
-from .util import add_args_to_init_call
+from iommi.declarative.namespace import Namespace
+from iommi.declarative.util import add_args_to_init_call
+from iommi.struct import Struct
 
 
 def with_meta(class_to_decorate=None, add_init_kwargs=True):
@@ -25,6 +26,9 @@ def with_meta(class_to_decorate=None, add_init_kwargs=True):
         add_args_to_init_call(class_to_decorate, get_extra_args_function, True)
 
     setattr(class_to_decorate, 'get_meta', classmethod(get_meta))
+    setattr(class_to_decorate, 'get_meta_flat', classmethod(get_meta_flat))
+    setattr(class_to_decorate, '__iommi_with_meta', True)
+    setattr(class_to_decorate, '__iommi_with_meta_add_init_kwargs', add_init_kwargs)
 
     return class_to_decorate
 
@@ -40,8 +44,22 @@ def get_meta(cls):
     merged_attributes = Namespace()
     for class_ in reversed(cls.mro()):
         if hasattr(class_, 'Meta'):
-            for key in class_.Meta.__dict__:
-                if not key.startswith('__'):
-                    value = getattr(class_.Meta, key)
-                    merged_attributes.setitem_path(key, value)
+            for meta_class_ in reversed(class_.Meta.mro()):
+                for key in meta_class_.__dict__:
+                    if not key.startswith('__'):
+                        value = getattr(class_.Meta, key)
+                        merged_attributes.setitem_path(key, value)
     return merged_attributes
+
+
+def get_meta_flat(cls):
+    attributes = Struct()
+    for class_ in reversed(cls.mro()):
+        if hasattr(class_, 'Meta'):
+            for meta_class_ in reversed(class_.Meta.mro()):
+                for key in meta_class_.__dict__:
+                    if not key.startswith('__'):
+                        value = getattr(class_.Meta, key)
+                        attributes[key] = value
+
+    return attributes
