@@ -5,19 +5,22 @@ from iommi.base import (
     keys,
 )
 from iommi.declarative.namespace import (
-    func_from_namespace,
     Namespace,
+    func_from_namespace,
 )
 
 _matches_cache = {}
+_use_cache = True
 
 
 def matches(caller_parameters, callee_parameters, __match_empty=False):
     cache_key = ';'.join((caller_parameters, callee_parameters, str(int(__match_empty))))  # pragma: no mutate
     # (mutation changes this to cached_value = None, which just slows down the code)
-    cached_value = _matches_cache.get(cache_key, None)  # pragma: no mutate
-    if cached_value is not None:
-        return cached_value
+    global _use_cache
+    if _use_cache:
+        cached_value = _matches_cache.get(cache_key, None)  # pragma: no mutate
+        if cached_value is not None:
+            return cached_value
 
     caller = set(caller_parameters.split(',')) if caller_parameters else set()
 
@@ -34,9 +37,9 @@ def matches(caller_parameters, callee_parameters, __match_empty=False):
         else:
             result = required <= caller <= required.union(optional)
 
-    _matches_cache[
-        cache_key
-    ] = result  # pragma: no mutate (mutation changes result to None which just makes things slower)
+    _matches_cache[cache_key] = (
+        result  # pragma: no mutate (mutation changes result to None which just makes things slower)
+    )
     return result
 
 
@@ -48,7 +51,9 @@ def get_callable_description(c):
             return 'lambda found at: `{}`'.format(inspect.getsource(c).strip())
         except OSError:  # pragma: no cover
             pass
-    return f'`{c}`'
+    if isinstance(c, Namespace):
+        return f'`{c}`'
+    return f'{c.__module__}.{c.__name__}'
 
 
 def is_callable(v):
@@ -58,7 +63,7 @@ def is_callable(v):
         return callable(v)
 
 
-def evaluate(func_or_value, __signature=None, __strict=False, __match_empty=True, **kwargs):
+def evaluate(func_or_value, *, __signature=None, __strict=False, __match_empty=True, **kwargs):
     if is_callable(func_or_value):
         if __signature is None:
             __signature = signature_from_kwargs(kwargs)
@@ -68,20 +73,23 @@ def evaluate(func_or_value, __signature=None, __strict=False, __match_empty=True
             return func_or_value(**kwargs)
 
         if __strict:
+            arguments = '\n        '.join(keys(kwargs))
+            parameters = '\n        '.join(inspect.getfullargspec(func_or_value)[0])
             assert isinstance(func_or_value, Namespace) and 'call_target' not in func_or_value, (
-                "Evaluating {} didn't resolve it into a value but strict mode was active, "
-                "the signature doesn't match the given parameters. "
-                "We had these arguments: {}".format(
-                    get_callable_description(func_or_value),
-                    ', '.join(keys(kwargs)),
-                )
-            )
+f'''Evaluating {get_callable_description(func_or_value)} didn't resolve it into a value but strict mode was active. The signature doesn't match the given parameters.
+
+    Possible inputs:
+        {arguments}
+
+    Function inputs:
+        {parameters}
+''')
     return func_or_value
 
 
-def evaluate_strict(func_or_value, __signature=None, __match_empty=True, **kwargs):
+def evaluate_strict(__func_or_value, *, __signature=None, __match_empty=True, **kwargs):
     # noinspection PyArgumentEqualDefault
-    return evaluate(func_or_value, __signature=None, __strict=True, __match_empty=__match_empty, **kwargs)
+    return evaluate(__func_or_value, __signature=None, __strict=True, __match_empty=__match_empty, **kwargs)
 
 
 def get_signature(func):
@@ -133,11 +141,11 @@ def evaluate_members(obj, **kwargs):
         evaluate_member(obj, key, **kwargs)
 
 
-def evaluate_member(obj, key, strict=True, **kwargs):
-    value = getattr(obj, key)
-    new_value = evaluate(value, __strict=strict, __signature=signature_from_kwargs(kwargs), **kwargs)
+def evaluate_member(__obj, __key, __strict=True, **kwargs):
+    value = getattr(__obj, __key)
+    new_value = evaluate(value, __strict=__strict, __signature=signature_from_kwargs(kwargs), **kwargs)
     if new_value is not value:
-        setattr(obj, key, new_value)
+        setattr(__obj, __key, new_value)
 
 
 def find_static_items(d):
@@ -145,6 +153,34 @@ def find_static_items(d):
         object.__setattr__(d, '_static_items', {k for k, v in items(d) if not callable(v)})
 
 
+def find_static_items_recursively(d):
+    if d and isinstance(d, Namespace):
+        find_static_items(d)
+        for k, v in items(d):
+            if isinstance(v, Namespace):
+                find_static_items_recursively(v)
+
+
 def evaluate_as_needed(d, kwargs, ignore=()):
     static_items = getattr(d, '_static_items', [])
     return {k: d[k] if k in static_items else evaluate_strict(v, **kwargs) for k, v in items(d) if k not in ignore}
+
+
+def evaluate_as_needed_recursively(d, kwargs, ignore=()):
+    static_items = getattr(d, '_static_items', [])
+
+    def value(k, v):
+        if k in static_items:
+            return d[k]
+        if isinstance(v, Namespace):
+            return evaluate_as_needed_recursively(v, kwargs, ignore)
+        return evaluate_strict(v, **kwargs)
+
+    return Namespace({
+        k: value(k, v)
+        for k, v in items(d) if k not in ignore
+    })
+
+
+def has_catch_all_kwargs(callback):
+    return get_signature(callback).endswith('|*')
