@@ -4,18 +4,18 @@ from typing import (
 )
 
 from iommi._web_compat import (
+    Template,
     format_html,
     render_template,
-    Template,
 )
 from iommi.attrs import (
     Attrs,
     render_attrs,
 )
 from iommi.base import (
-    capitalize,
     MISSING,
     NOT_BOUND_MESSAGE,
+    capitalize,
     values,
 )
 from iommi.declarative.dispatch import dispatch
@@ -23,27 +23,26 @@ from iommi.declarative.namespace import (
     EMPTY,
     Namespace,
 )
-from iommi.declarative.with_meta import with_meta
 from iommi.evaluate import (
     evaluate_as_needed,
     evaluate_strict,
     find_static_items,
 )
 from iommi.member import (
+    bind_member,
     bind_members,
     refine_done_members,
 )
 from iommi.part import (
-    as_html,
     Part,
     PartType,
+    as_html,
 )
 
 # https://html.spec.whatwg.org/multipage/syntax.html#void-elements
 from iommi.refinable import (
     EvaluatedRefinable,
     Prio,
-    Refinable,
     RefinableMembers,
     SpecialEvaluatedRefinable,
 )
@@ -67,6 +66,50 @@ _void_elements = [
 ]
 
 
+class TransientFragment:
+    @dispatch(
+        attrs__class=EMPTY,
+        attrs__style=EMPTY,
+    )
+    def __init__(self, *, parent, template=None, attrs, children, tag):
+        self._is_bound = True
+        self.template = template
+        self.attrs = attrs
+        self.children = children
+        self.tag = tag
+        self.parent = parent
+
+    @property
+    def include(self):
+        return True
+
+    def get_request(self):
+        return self.parent.get_request()
+
+    def iommi_evaluate_parameters(self):
+        return self.parent.iommi_evaluate_parameters()
+
+    def get_context(self):
+        return self.parent.get_context()
+
+    def render_text_or_children(self, context=None):
+        if context is None:
+            context = self.get_context()
+        request = self.get_request()
+        return format_html(
+            '{}' * len(self.children),
+            *[as_html(part=x, context=context, request=request) for x in values(self.children)],
+        )
+
+    def __html__(self):
+        assert self._is_bound, NOT_BOUND_MESSAGE
+        context = {**self.get_context(), **self.iommi_evaluate_parameters()}
+        return fragment__render(
+            fragment=self,
+            context=context,
+        )
+
+
 def fragment__render(fragment, context):
     if not fragment.include:
         return ''
@@ -119,10 +162,11 @@ class Tag:
             return format_html('</{}>', self.tag)
 
 
-@with_meta
 class Fragment(Part, Tag):
     # language=rst
     """
+    .. _Fragment:
+
     `Fragment` is a class used to build small HTML fragments that plug into iommi's structure.
 
     .. code-block:: python
@@ -133,6 +177,9 @@ class Fragment(Part, Tag):
 
         h1 = Fragment(children__text='Tony', tag='h1')
 
+        # There is also a shorthand version that is identical to the above:
+        h1 = Fragment('Tony', tag='h1')
+
     It's easiest to use via the html builder:
 
     .. code-block:: python
@@ -264,11 +311,12 @@ def build_and_bind_h_tag(p):
             p.h_tag = p.h_tag(
                 _name='h_tag',
                 children__text=capitalize(evaluate_strict(p.title, **p.iommi_evaluate_parameters())),
-            ).bind(parent=p)
+            )
+            bind_member(p, name='h_tag')
         else:
             p.h_tag = ''
     elif p.h_tag is not None:
-        p.h_tag = p.h_tag.bind(parent=p)
+        bind_member(p, name='h_tag')
         p.h_tag._name = 'h_tag'
 
     if p.h_tag is None:
