@@ -1,34 +1,39 @@
 import warnings
 from typing import (
     Dict,
-    Iterator,
     List,
     Type,
 )
 
 from django.core.exceptions import FieldDoesNotExist
+from django.db.models import Field as DjangoField
 from django.db.models import (
-    Field as DjangoField,
     ManyToManyRel,
     ManyToOneRel,
     Model,
+    OneToOneRel,
 )
-from iommi.struct import Struct
+try:
+    from django.db.models import GeneratedField
+except ImportError:
+    GeneratedField = None
 
-from iommi.base import (
-    MISSING,
-)
+from iommi.base import MISSING
 from iommi.declarative.dispatch import dispatch
 from iommi.declarative.namespace import (
     Namespace,
     setdefaults_path,
 )
-from iommi.evaluate import evaluate
+from iommi.evaluate import (
+    evaluate,
+    evaluate_strict,
+)
 from iommi.refinable import (
     Refinable,
     RefinableObject,
     SpecialEvaluatedRefinable,
 )
+from iommi.struct import Struct
 
 
 def create_members_from_model(
@@ -50,27 +55,23 @@ def create_members_from_model(
         if exclude is not None and model_field_name in exclude:
             continue
         name = model_field_name.replace('__', '_')
-        definition = Namespace(
+        conf = Namespace(
             _name=name,
-            call_target__cls=member_class,
-            call_target__attribute='from_model',
             model_field_name=model_field_name,
             model=model,
         )
         if default_included is False:
             setdefaults_path(
-                definition,
+                conf,
                 include=False,
             )
         if include is not None and name in include:
             setdefaults_path(
-                definition,
+                conf,
                 include=True,
             )
 
-        member = definition()
-        if member is not None:
-            members[name] = member
+        members[name] = member_class._from_model(**conf)
 
     return members
 
@@ -105,7 +106,7 @@ def member_from_model(
                 model_field_name=field_path_rest,
                 **kwargs,
             )
-            result = result.refine(attr=model_field_name)
+            result.attr = model_field_name
             return result
     else:
         if model is None:
@@ -113,7 +114,11 @@ def member_from_model(
         if model_field_name is None:
             model_field_name = model_field.name
 
-    factory = factory_lookup.get(type(model_field), MISSING)
+    model_field_type = type(model_field)
+    if model_field_type == GeneratedField:
+        model_field_type = type(model_field.output_field)
+
+    factory = factory_lookup.get(model_field_type, MISSING)
 
     if factory is MISSING:
         for django_field_type, foo in reversed(list(factory_lookup.items())):
@@ -124,7 +129,7 @@ def member_from_model(
     if factory is MISSING:
 
         def no_factory_defined(**_):
-            message = f'No factory for {model.__name__}.{model_field_name} of type {type(model_field).__name__}.'
+            message = f'No factory for {model.__name__}.{model_field_name} of type {model_field_type.__name__}.'
             if factory_lookup_register_function is not None:
                 message += (
                     ' Register a factory with register_factory or %s, you can also register one that returns None to not handle this field type'
@@ -138,7 +143,17 @@ def member_from_model(
         return None
 
     # Not strict evaluate on purpose
-    factory = evaluate(factory, __match_empty=False, model_field=model_field, model_field_name=model_field_name)
+    if isinstance(factory, dict):
+        conf_or_instance = factory
+    else:
+        conf_or_instance = evaluate_strict(factory, model_field=model_field, model_field_name=model_field_name)
+
+    if isinstance(conf_or_instance, cls):
+        return conf_or_instance
+
+    assert isinstance(conf_or_instance, dict), f'Factories must return a configuration dict or an instance of {cls.__name__}. Got {type(conf_or_instance).__name__}: "{conf_or_instance}"'
+    conf = conf_or_instance
+    del conf_or_instance
 
     setdefaults_path(
         kwargs,
@@ -147,24 +162,21 @@ def member_from_model(
     )
 
     defaults = defaults_factory(model_field)
-    if isinstance(factory, Namespace):
-        factory = setdefaults_path(
-            Namespace(),
-            factory,
-            defaults,
-        )
-    else:
-        kwargs.update(**defaults)
 
-    return factory(model_field=model_field, model_field_name=model_field_name, model=model, **kwargs)
+    return setdefaults_path(
+        Namespace(),
+        kwargs,
+        conf,
+        defaults,
+        model_field=model_field,
+        model_field_name=model_field_name,
+        model=model,
+    )
 
 
 def get_field_by_name(model: Type[Model]) -> Dict[str, DjangoField]:
     if not hasattr(model._meta, '_iommi_fields'):
-        model._meta._iommi_fields = {
-            get_field_name(field): field
-            for field in model._meta.get_fields()
-        }
+        model._meta._iommi_fields = {get_field_name(field): field for field in model._meta.get_fields()}
         if None in model._meta._iommi_fields:
             model._meta._iommi_fields.pop(None)
     return model._meta._iommi_fields
@@ -174,10 +186,14 @@ def get_field_name(field: DjangoField) -> str:
     if isinstance(field, ManyToManyRel):
         return field.related_name
     elif isinstance(field, ManyToOneRel):
-        if field.related_name:
-            return field.related_name
-        elif field.related_name == '+':
+        if field.related_name == '+':
             return None
+        elif field.related_query_name:
+            return field.related_query_name
+        elif field.related_name:
+            return field.related_name
+        elif isinstance(field, OneToOneRel):
+            return field.name
         else:
             return f'{field.name}_set'
     else:
@@ -192,7 +208,9 @@ def get_field(model: Type[Model], field_name: str) -> DjangoField:
         return get_field_by_name(model)[field_name]
     except KeyError:
         valid_names = '\n    '.join(sorted(get_field_by_name(model).keys()))
-        raise FieldDoesNotExist(f"{model._meta.object_name} has no field with name '{field_name}', valid names are:\n\n    {valid_names}")
+        raise FieldDoesNotExist(
+            f"{model._meta.object_name} has no field with name '{field_name}', valid names are:\n\n    {valid_names}"
+        )
 
 
 def get_field_path(model, path):
@@ -219,10 +237,7 @@ def check_list(model: Type[Model], paths: List[str], operation: str) -> None:
                 current_model = get_field(current_model, part).remote_field.model
             except FieldDoesNotExist:
                 return sorted(
-                    [
-                        '__'.join(prefix + [name])
-                        for name in get_field_by_name(current_model).keys()
-                    ]
+                    ['__'.join(prefix + [name]) for name in get_field_by_name(current_model).keys()]
                     + ['__'.join(prefix + ['pk'])]
                 )
             else:
@@ -308,4 +323,9 @@ class AutoConfig(RefinableObject):
 
     @dispatch
     def __init__(self, **kwargs):
+        """
+        :param model: A Django model class
+        :param include: A list of attribute names to include, use `__` to drill down to nested attributes. Example: `['album', 'album__year']`
+        :param exclude: A list of attribute names to exclude, use `__` to drill down to nested attributes. Example: `['album', 'album__year']`
+        """
         super(AutoConfig, self).__init__(**kwargs)
