@@ -4,8 +4,6 @@ from typing import (
     Type,
 )
 
-from iommi.struct import Struct
-
 from iommi.base import (
     items,
     keys,
@@ -18,8 +16,10 @@ from iommi.declarative.namespace import (
 from iommi.refinable import (
     Prio,
     RefinableMembers,
+    RefinableObject,
 )
 from iommi.sort_after import sort_after
+from iommi.struct import Struct
 from iommi.traversable import (
     Traversable,
 )
@@ -43,6 +43,18 @@ class Members(Traversable):
         self._bound_members = MemberBinder(self, self._declared_members, self._unknown_types_fall_through)
 
 
+def reify_conf(conf):
+    if isinstance(conf, Namespace):
+        missing = object()
+        iommi_shortcut_stack = conf.pop('iommi_shortcut_stack', missing)
+        r = conf()
+        if iommi_shortcut_stack is not missing:
+            assert not getattr(r, 'iommi_shortcut_stack', [])
+            r.iommi_shortcut_stack = iommi_shortcut_stack
+        return r
+    return conf
+
+
 def refine_done_members(
     container,
     *,
@@ -119,7 +131,7 @@ def refine_done_members(
                         call_target__cls=cls,
                         _name=key,
                     )
-                    member_by_name[key] = item()
+                    member_by_name[key] = reify_conf(item)
             else:
                 assert (
                     unknown_types_fall_through or item is None
@@ -127,9 +139,12 @@ def refine_done_members(
                 member_by_name[key] = item
 
     for k, v in items(Namespace(_unapplied_config)):
-        member_by_name[k] = member_by_name[k].refine(Prio.member, **v)
-        # noinspection PyProtectedMember
-        assert member_by_name[k]._name is not None
+        m = member_by_name[k]
+        if isinstance(m, Namespace):
+            member_by_name[k] = setdefaults_path(Namespace(), v, m)
+        else:
+            assert isinstance(m, RefinableObject)
+            member_by_name[k] = m.refine(Prio.member, **v)
 
     if extra_member_defaults:
         for k, v in items(Namespace(extra_member_defaults)):
@@ -142,7 +157,11 @@ def refine_done_members(
                     v,
                     call_target__cls=cls,
                     _name=k,
-                )()
+                )
+
+    for k, v in items(member_by_name):
+        member_by_name[k] = reify_conf(v)
+        if isinstance(member_by_name[k], Traversable):
             # noinspection PyProtectedMember
             assert member_by_name[k]._name is not None
 
@@ -184,6 +203,13 @@ def bind_members(container: Traversable, *, name: str, lazy=True) -> None:
         _force_bind_all(m._bound_members)
 
 
+def bind_member(container, *, name: str) -> None:
+    bound_member = getattr(container, name).bind(parent=container)
+    setattr(container, name, bound_member)
+    if bound_member is not None:
+        setattr(container._bound_members, name, bound_member)
+
+
 class ForbiddenNamesException(Exception):
     pass
 
@@ -228,9 +254,7 @@ class MemberBinder(dict):
     def __delitem__(self, name):
         dict.__delitem__(self, name)
         _bindable_names = object.__getattribute__(self, '_bindable_names')
-        _declared_members = object.__getattribute__(self, '_declared_members')
         _bindable_names.remove(name)
-        del _declared_members[name]
 
     def get(self, name, *args):
         _force_bind(self, name)
@@ -264,7 +288,6 @@ class MemberBinder(dict):
 # noinspection PyCallByClass
 def _force_bind(member_binder: MemberBinder, name: str):
     if name not in member_binder:
-
         _parent = object.__getattribute__(member_binder, '_parent')
         _bindable_names = object.__getattribute__(member_binder, '_bindable_names')
         _declared_members = object.__getattribute__(member_binder, '_declared_members')
