@@ -8,13 +8,10 @@ from iommi._web_compat import (
     format_html,
     template_types,
 )
-from iommi.evaluate import (
-    evaluate_as_needed,
-    find_static_items,
-)
 from iommi.base import (
     build_as_view_wrapper,
     items,
+    NOT_BOUND_MESSAGE,
     values,
 )
 from iommi.declarative import declarative
@@ -23,20 +20,23 @@ from iommi.declarative.namespace import (
     EMPTY,
     Namespace,
 )
-from iommi.declarative.with_meta import with_meta
+from iommi.evaluate import (
+    evaluate_as_needed,
+    find_static_items,
+)
 from iommi.fragment import (
-    build_and_bind_h_tag,
     Fragment,
     Header,
+    build_and_bind_h_tag,
 )
 from iommi.member import (
     bind_members,
     refine_done_members,
 )
 from iommi.part import (
-    as_html,
     Part,
     PartType,
+    as_html,
 )
 from iommi.refinable import (
     EvaluatedRefinable,
@@ -49,7 +49,6 @@ from iommi.sort_after import sort_after
 from iommi.traversable import Traversable
 
 
-@with_meta
 @declarative(
     parameter='parts_dict',
     is_member=lambda obj: isinstance(obj, (Part, str) + template_types),
@@ -60,7 +59,7 @@ class Page(Part):
     """
     A page is used to compose iommi parts into a bigger whole.
 
-    See the `howto <https://docs.iommi.rocks/en/latest/cookbook_parts_pages.html#parts-pages>`_ for example usages.
+    See the `howto <https://docs.iommi.rocks//cookbook_parts_pages.html#parts-pages>`_ for example usages.
     """
 
     title: str = EvaluatedRefinable()
@@ -99,7 +98,7 @@ class Page(Part):
             name='parts',
             members_from_namespace=self.parts,
             members_from_declared=_parts_dict,
-            cls=self.get_meta().member_class,
+            cls=self.member_class,
         )
         find_static_items(self.context)
         super(Page, self).on_refine_done()
@@ -116,6 +115,7 @@ class Page(Part):
 
     @dispatch(render=lambda rendered: format_html('{}' * len(rendered), *values(rendered)))
     def __html__(self, *, render=None):
+        assert self._is_bound, NOT_BOUND_MESSAGE
         self.context = evaluate_as_needed(self.context or {}, self.iommi_evaluate_parameters())
         request = self.get_request()
         context = {**self.get_context(), **self.iommi_evaluate_parameters()}
