@@ -1,10 +1,13 @@
 import operator
+from datetime import datetime
 from functools import reduce
 from typing import (
+    Callable,
     Type,
     Union,
 )
 
+from django.conf import settings
 from django.core.exceptions import (
     MultipleObjectsReturned,
     ObjectDoesNotExist,
@@ -15,7 +18,11 @@ from django.db.models import (
     Q,
     QuerySet,
 )
-from django.utils.translation import gettext
+from django.utils import timezone
+from django.utils.translation import (
+    gettext_lazy,
+    pgettext,
+)
 from pyparsing import (
     alphanums,
     alphas,
@@ -29,13 +36,13 @@ from pyparsing import (
     QuotedString,
     quotedString,
     Word,
-    ZeroOrMore,
+    ZeroOrMore
 )
 
 from iommi._web_compat import (
     render_template,
     Template,
-    ValidationError,
+    ValidationError
 )
 from iommi.action import (
     Action,
@@ -46,7 +53,7 @@ from iommi.base import (
     MISSING,
     model_and_rows,
     NOT_BOUND_MESSAGE,
-    values,
+    values
 )
 from iommi.declarative import declarative
 from iommi.declarative.dispatch import dispatch
@@ -54,9 +61,8 @@ from iommi.declarative.namespace import (
     EMPTY,
     getattr_path,
     Namespace,
-    setdefaults_path,
+    setdefaults_path
 )
-from iommi.declarative.with_meta import with_meta
 from iommi.endpoint import path_join
 from iommi.evaluate import (
     evaluate,
@@ -66,11 +72,10 @@ from iommi.form import (
     bool_parse,
     boolean_tristate__parse,
     date_parse,
-    datetime_parse,
     float_parse,
     Form,
     int_parse,
-    time_parse,
+    time_parse
 )
 from iommi.fragment import (
     Fragment,
@@ -80,15 +85,16 @@ from iommi.from_model import (
     create_members_from_model,
     get_search_fields,
     member_from_model,
-    NoRegisteredSearchFieldException,
+    NoRegisteredSearchFieldException
 )
 from iommi.member import (
+    bind_member,
     bind_members,
     refine_done_members,
+    reify_conf,
 )
 from iommi.part import (
     Part,
-    request_data,
 )
 from iommi.refinable import (
     EvaluatedRefinable,
@@ -96,7 +102,7 @@ from iommi.refinable import (
     Refinable,
     refinable,
     RefinableMembers,
-    SpecialEvaluatedRefinable,
+    SpecialEvaluatedRefinable
 )
 from iommi.shortcut import (
     Shortcut,
@@ -145,10 +151,12 @@ def to_string_surrounded_by_quote(v):
 
 
 def value_to_str_for_query(filter, v):
-    if type(v) == bool:
+    if isinstance(v, bool):
         return {True: '1', False: '0'}.get(v)
     if type(v) in (int, float):
         return str(v)
+    if isinstance(v, datetime) and timezone.is_aware(v):
+        v = timezone.make_naive(v, timezone=timezone.utc)
     if isinstance(v, Model):
         model = type(v)
         search_field = filter.search_fields[-1]
@@ -163,11 +171,7 @@ def value_to_str_for_query(filter, v):
 
 def build_query_expression(*, filter, value):
     if isinstance(value, Model):
-        try:
-            # We ignore the return value on purpose here. We are after the raise.
-            get_search_fields(model=type(value))
-        except NoRegisteredSearchFieldException:
-            return f'{filter.query_name}.pk={value.pk}'
+        return f'{filter.query_name}.pk={value.pk}'
 
     return f'{filter.query_name}{filter.query_operator_for_field}{value_to_str_for_query(filter, value)}'
 
@@ -182,7 +186,7 @@ def boolean__query_operator_to_q_operator(op):
     return 'exact'
 
 
-def choice_queryset_value_to_q(filter, op, value_string_or_f):
+def choice_queryset_value_to_q(filter, op, value_string_or_f, **_):
     if op != '=':
         raise QueryException(f'Invalid operator "{op}" for filter "{filter._name}"')
     if filter.attr is None:
@@ -194,6 +198,7 @@ def choice_queryset_value_to_q(filter, op, value_string_or_f):
         for search_field in filter.search_fields:
             try:
                 instance = filter.choices.get(**{search_field: str(value_string_or_f)})
+                break
             except ObjectDoesNotExist:
                 pass
     except MultipleObjectsReturned:
@@ -220,7 +225,6 @@ def choice_queryset__is_valid_filter(name, filter, **_):
     )
 
 
-@with_meta
 class Filter(Part):
     """
     Class that describes a filter that you can search for.
@@ -240,6 +244,7 @@ class Filter(Part):
     unary = Refinable()
     is_valid_filter = Refinable()
     query_name = EvaluatedRefinable()
+    pk_lookup_to_q = Refinable()
 
     @with_defaults(
         query_operator_for_field='=',
@@ -286,9 +291,10 @@ class Filter(Part):
         return dict(filter=self)
 
     def __html__(self, *, render=None):
-        assert (
-            False
-        ), "Filters aren't rendered directly. You either render the Field corresponding to the filter, or the entire Query object."
+        assert False, (
+            "Filters aren't rendered directly. You either render the Field corresponding to the filter, "
+            "or the entire Query object."
+        )
 
     @staticmethod
     @refinable
@@ -302,7 +308,7 @@ class Filter(Part):
 
     @staticmethod
     @refinable
-    def value_to_q(filter, op, value_string_or_f) -> Q:
+    def value_to_q(filter, op, value_string_or_f, **_) -> Q:
         if filter.attr is None:
             return Q()
         negated = False
@@ -322,7 +328,13 @@ class Filter(Part):
             return r
 
     @classmethod
-    def from_model(cls, model, model_field_name=None, model_field=None, **kwargs):
+    @dispatch
+    def from_model(cls, model=None, model_field_name=None, model_field=None, **kwargs):
+        return reify_conf(cls._from_model(model=model, model_field_name=model_field_name, model_field=model_field, **kwargs))
+
+    @classmethod
+    @dispatch
+    def _from_model(cls, model, model_field_name=None, model_field=None, **kwargs):
         return member_from_model(
             cls=cls,
             model=model,
@@ -360,7 +372,6 @@ class Filter(Part):
     def choice(cls, **kwargs):
         """
         Field that has one value out of a set.
-        :type choices: list
         """
         instance = cls(**kwargs)
         instance = instance.refine(
@@ -376,7 +387,6 @@ class Filter(Part):
     def multi_choice(cls, **kwargs):
         """
         Field that has one value out of a set.
-        :type choices: list
         """
         instance = cls(**kwargs)
         instance = instance.refine(
@@ -385,6 +395,10 @@ class Filter(Part):
         )
         return instance
 
+    @classmethod
+    def checkboxes(cls, **kwargs):
+        return cls.multi_choice(**kwargs)
+
     @classmethod
     @with_defaults(
         field__call_target__attribute='choice_queryset',
@@ -392,7 +406,7 @@ class Filter(Part):
         value_to_q=choice_queryset_value_to_q,
         is_valid_filter=choice_queryset__is_valid_filter,
     )
-    def choice_queryset(cls, choices: QuerySet = None, **kwargs):
+    def choice_queryset(cls, choices: QuerySet | Callable[..., QuerySet] = None, **kwargs):
         """
         Field that has one value out of a set.
         """
@@ -442,6 +456,7 @@ class Filter(Part):
     @classmethod
     @with_defaults(
         field__call_target__attribute='number',
+        query_operator_to_q_operator=case_sensitive_query_operator_to_q_operator,
     )
     def number(cls, **kwargs):
         return cls(**kwargs)
@@ -479,10 +494,13 @@ class Filter(Part):
 
     @classmethod
     @with_defaults(
-        field__call_target__attribute='datetime',
-        parse=datetime_parse,
+        field__call_target__attribute='date',
+        parse=date_parse,
+        extra_evaluated__is_tz_aware=lambda **_: settings.USE_TZ,
     )
     def datetime(cls, **kwargs):
+        if isinstance(kwargs.get('attr'), str):
+            kwargs['attr'] = kwargs['attr'] + '__date'
         return cls(**kwargs)
 
     @classmethod
@@ -514,6 +532,13 @@ class Filter(Part):
     def file(cls, **kwargs):
         return cls(**kwargs)
 
+    @classmethod
+    @with_defaults(
+        field__call_target__attribute='duration',
+    )
+    def duration(cls, **kwargs):
+        return cls.text(**kwargs)
+
     @classmethod
     @with_defaults(
         field__call_target__attribute='foreign_key',
@@ -577,6 +602,9 @@ def default_endpoint__errors(query, **_):
 
 
 class QueryAutoConfig(AutoConfig):
+    """
+    :param rows: A `QuerySet` object. If this field is specified, the `model` attribute will be automatically derived. This cannot be a callable, in that case set `model` and use `rows=lambda...` instead of `auto__rows`.
+    """
     rows = Refinable()
 
 
@@ -595,7 +623,7 @@ class Advanced(Fragment):
     def on_refine_done(self):
         self.toggle = self.toggle(
             _name='toggle',
-            display_name=gettext('Switch to advanced search'),
+            display_name=gettext_lazy('Switch to advanced search'),
         ).refine_done(parent=self)
         super(Advanced, self).on_refine_done()
 
@@ -605,7 +633,6 @@ class Advanced(Fragment):
 
 
 @declarative(Filter, '_filters_dict', add_init_kwargs=False)
-@with_meta
 class Query(Part):
     # language=rst
     """
@@ -622,7 +649,7 @@ class Query(Part):
         )
     """
 
-    auto: Namespace = Refinable()
+    auto: QueryAutoConfig = Refinable()
     form: Namespace = Refinable()
     advanced: Namespace = Refinable()
     model: Type[Model] = SpecialEvaluatedRefinable()
@@ -630,8 +657,8 @@ class Query(Part):
     template: Union[str, Template] = EvaluatedRefinable()
     form_container: Fragment = EvaluatedRefinable()
 
-    member_class = Refinable()
-    form_class = Refinable()
+    member_class: Type[Filter] = Refinable()
+    form_class: Type[Form] = Refinable()
 
     # Filters need to be at the end to not steal the short names
     filters: Namespace = RefinableMembers()
@@ -663,6 +690,7 @@ class Query(Part):
                 rows=auto.rows,
                 include=auto.include,
                 exclude=auto.exclude,
+                default_included=auto.default_included,
             )
 
             assert self.model is None, (
@@ -689,7 +717,7 @@ class Query(Part):
             members_from_namespace=self.filters,
             members_from_declared=self.get_declared('_filters_dict'),
             members_from_auto=filters_from_auto,
-            cls=self.get_meta().member_class,
+            cls=self.member_class,
         )
 
         self._on_refine_done_form()
@@ -703,6 +731,7 @@ class Query(Part):
         render__call_target=render_template,
     )
     def __html__(self, *, render=None):
+        assert self._is_bound, NOT_BOUND_MESSAGE
         if not self.iommi_bound_members().filters.iommi_bound_members():
             return ''
 
@@ -715,23 +744,21 @@ class Query(Part):
         return render(request=self.get_request())
 
     def _on_refine_done_form(self):
-
-        field_class = self.get_meta().form_class.get_meta().member_class
+        field_class = self.form_class.get_meta().member_class
 
         declared_fields = Struct()
 
         freetext_search_config = self.iommi_namespace.form.get('fields', {}).get(FREETEXT_SEARCH_NAME, {})
         if freetext_search_config is not None:
-            declared_fields[FREETEXT_SEARCH_NAME] = field_class(
-                **setdefaults_path(
-                    Namespace(),
-                    freetext_search_config,
-                    _name=FREETEXT_SEARCH_NAME,
-                    display_name=gettext('Search'),
-                    required=False,
-                    include=lambda query, **_: any(filter.freetext for filter in values(query.filters)),
-                    help__include=False,
-                )
+            declared_fields[FREETEXT_SEARCH_NAME] = setdefaults_path(
+                Namespace(),
+                freetext_search_config,
+                _name=FREETEXT_SEARCH_NAME,
+                call_target__cls=field_class,
+                display_name=gettext_lazy('Search'),
+                required=False,
+                include=lambda query, **_: any(filter.freetext for filter in values(query.filters)),
+                help__include=False,
             )
 
         for name, filter in items(self.iommi_namespace.filters):
@@ -768,8 +795,7 @@ class Query(Part):
 
         # noinspection PyCallingNonCallable
         self.form: Form = (
-            self.get_meta()
-            .form_class(
+            self.form_class(
                 **setdefaults_path(
                     Namespace(),
                     form_args,
@@ -778,7 +804,7 @@ class Query(Part):
                     attrs__method='get',
                     actions__submit=dict(
                         attrs={'data-iommi-filter-button': ''},
-                        display_name=gettext('Filter'),
+                        display_name=pgettext(context='verb', message='Filter'),
                     ),
                 )
             )
@@ -793,16 +819,13 @@ class Query(Part):
         bind_members(self, name='filters')
 
         request = self.get_request()
-        self.query_advanced_value = request_data(request).get(self.get_advanced_query_param(), '') if request else ''
+        self.query_advanced_value = request.GET.get(self.get_advanced_query_param(), '').strip() if request else ''
 
         bind_members(self, name='endpoints')
 
-        self.form = self.form.bind(parent=self)
-        self._bound_members.form = self.form
-
-        self.advanced = self._bound_members.advanced = self.advanced.bind(parent=self)
-
-        self.form_container = self.form_container.bind(parent=self)
+        bind_member(self, name='form')
+        bind_member(self, name='advanced')
+        bind_member(self, name='form_container')
 
         self.filter_name_by_query_name = {x.query_name: name for name, x in items(self.filters)}
 
@@ -814,6 +837,7 @@ class Query(Part):
             try:
                 q = query.get_q()
             except QueryException:
+                # This exception is dumped into `query_error` inside `get_q` before being reraised, so it's fine to ignore it here.
                 pass
             if q:
                 rows = rows.filter(q)
@@ -954,7 +978,7 @@ class Query(Part):
                 raise QueryException(
                     f'"{filter_name}" is not a unary filter, you must use it like "{filter_name}=something"'
                 )
-            result = filter.value_to_q(filter=filter, op='=', value_string_or_f=value)
+            result = filter.invoke_callback(filter.value_to_q, op='=', value_string_or_f=value)
             return result
         raise QueryException(
             f'Unknown unary filter "{filter_name}", available filters: {", ".join(list(keys(self.filters)))}'
@@ -991,9 +1015,15 @@ class Query(Part):
             try:
                 pk = int(value_string_or_filter_name)
             except ValueError:
-                raise QueryException(f'Could not interpret {value_string_or_filter_name} as an integer')
+                pk = value_string_or_filter_name  # pk might be non-int
 
-            return Q(**{f'{filter.attr}__pk': pk})
+            if filter.pk_lookup_to_q:
+                return filter.invoke_callback(filter.pk_lookup_to_q, pk=pk)
+            else:
+                if filter.attr is not None:
+                    return Q(**{f'{filter.attr}__pk': pk})
+                else:
+                    return Q(pk=pk)
 
         if (
             isinstance(value_string_or_filter_name, str)
@@ -1004,9 +1034,11 @@ class Query(Part):
         else:
             value_string_or_f = value_string_or_filter_name
         try:
-            result = filter.value_to_q(filter=filter, op=op, value_string_or_f=value_string_or_f)
+            result = filter.invoke_callback(filter.value_to_q, op=op, value_string_or_f=value_string_or_f)
         except ValidationError as e:
             raise QueryException(f'{e.message}')
+        except ValueError as e:
+            raise QueryException(f'Invalid value for filter "{query_name}": {e}')
         if result is None:
             raise QueryException(f'Unknown value "{value_string_or_f}" for filter "{query_name}"')
         return result
@@ -1036,8 +1068,8 @@ class Query(Part):
         if request is None:
             return ''
 
-        if request_data(request).get(self.get_advanced_query_param(), '').strip():
-            return request_data(request).get(self.get_advanced_query_param())
+        if self.query_advanced_value:
+            return self.query_advanced_value
         elif form.is_valid():
 
             def expr(field, is_list, value):
@@ -1096,7 +1128,7 @@ class Query(Part):
 
     @classmethod
     @dispatch()
-    def _from_model(cls, *, rows=None, model=None, include=None, exclude=None):
+    def _from_model(cls, *, rows=None, model=None, include=None, exclude=None, default_included=False):
         assert rows is None or isinstance(rows, QuerySet), (
             'auto__rows needs to be a QuerySet for filter generation to work. '
             'If it needs to be a lambda, provide a model with auto__model for filter generation, '
@@ -1105,5 +1137,5 @@ class Query(Part):
 
         model, rows = model_and_rows(model, rows)
         assert model is not None or rows is not None, "auto__model or auto__rows must be specified"
-        filters = cls.filters_from_model(model=model, include=include, exclude=exclude)
+        filters = cls.filters_from_model(model=model, include=include, exclude=exclude, default_included=default_included)
         return model, rows, filters
