@@ -1,8 +1,7 @@
+import warnings
 from copy import copy
-from enum import (
-    auto,
-    Enum,
-)
+from enum import auto, \
+    Enum
 from typing import (
     Any,
     List,
@@ -16,6 +15,10 @@ from iommi.declarative.namespace import (
     getattr_path,
     Namespace,
 )
+from iommi.declarative.with_meta import (
+    get_meta,
+    get_meta_flat,
+)
 from iommi.evaluate import get_signature
 
 
@@ -48,6 +51,7 @@ class Prio(Enum):
     constructor = auto()
     shortcut = auto()
     style = auto()
+    meta = auto()
     base = auto()
     member = auto()
     refine = auto()
@@ -163,17 +167,20 @@ def get_evaluated_attributes(result):
 class SpecialEvaluatedRefinable(Refinable):
     pass
 
+
 def special_evaluated_refinable(f):
     f = refinable(f)
     f.__iommi__special_evaluated = True
     return f
 
+
 def is_special_evaluated_refinable(x):
     return isinstance(x, SpecialEvaluatedRefinable) or getattr(x, '__iommi__special_evaluated', False)
 
 
 _get_special_evaluated_attributes_cache = {}
 
+
 def get_special_evaluated_attributes(result):
     class_ = type(result)
     try:
@@ -215,12 +222,29 @@ class RefinableObject:
                 + '\n'
             )
 
+    @classmethod
+    def get_meta(cls):
+        return get_meta(cls)
+
+    @classmethod
+    def get_meta_flat(cls):
+        return get_meta_flat(cls)
+
     def refine_done(self, parent=None):
         result = copy(self)
         del self
 
         assert not result.is_refine_done, f"refine_done() already invoked on {result!r}"
 
+        meta_params = result.get_meta()
+        if meta_params:
+            if getattr(result, '__iommi_with_meta', False):
+                warnings.warn(
+                    f'RefinableObject {result.__class__} should not merge class Meta attributes into the constructor invocation. '
+                    f'Drop @with_meta decorator.'
+                )
+            result.iommi_namespace = result.iommi_namespace._refine(Prio.meta, **meta_params)
+
         if hasattr(result, 'apply_style'):
             is_root = parent is None
             enclosing_style = None if is_root else parent.iommi_style
@@ -259,12 +283,13 @@ class RefinableObject:
 
         result.on_refine_done()
 
-        refinables_dynamic = set()
+        # dict instead of set() to respect order
+        refinables_dynamic = dict()
         for k in get_evaluated_attributes(result):
             v = getattr(result, k)
             if callable(v):
                 get_signature(v)  # warm the signature cache
-                refinables_dynamic.add(k)
+                refinables_dynamic[k] = True
         result._refinables_dynamic = refinables_dynamic
         return result
 
