@@ -1,7 +1,6 @@
 import itertools
 import linecache
 import logging
-import os
 import re
 import sys
 import threading
@@ -14,27 +13,29 @@ from datetime import (
 from logging import addLevelName
 from time import monotonic
 
+from django.conf import settings
 from django.db import connections
+from django.db.backends import utils as django_db_utils
 from django.db.utils import DEFAULT_DB_ALIAS
 from django.http import HttpResponse
-from django.utils.html import format_html
+from django.utils.timezone import now
 
+from iommi._web_compat import format_html
 from iommi.attrs import render_style
 from iommi.thread_locals import (
     get_current_request,
     set_current_request,
 )
 
+
+def no_coloring(text, color=None, on_color=None, attrs=None):
+    return text
+
+
 try:
     from termcolor import colored
 except ImportError:
-
-    def colored(text, color=None, on_color=None, attrs=None):
-        return text
-
-
-from django.conf import settings
-from django.db.backends import utils as django_db_utils
+    colored = no_coloring
 
 
 log = logging.getLogger('db')
@@ -88,7 +89,7 @@ class Middleware:
 
     def __call__(self, request):
         set_current_request(request)
-        request.iommi_start_time = datetime.now()
+        request.iommi_start_time = now()
         sql_trace = request.GET.get('_iommi_sql_trace')
 
         if sql_trace is None:
@@ -119,25 +120,35 @@ class Middleware:
                 if iommi_sql_debug_log is not None:
                     total_duration = float(sum(x['duration'] for x in iommi_sql_debug_log))
                     result = [
+                        # language=HTML
                         '''
-                        <style>
-                            a, span {
-                                box-sizing: border-box;
-                            }
-                            @media (prefers-color-scheme: dark) {
-                                html {
-                                    background-color: black;
-                                    color: #bbb;
+                            <style>
+                                a, span {
+                                    box-sizing: border-box;
                                 }
-                                b {
-                                    color: white;
+                                @media (prefers-color-scheme: dark) {
+                                    html {
+                                        background-color: black;
+                                        color: #bbb;
+                                    }
+                                    b {
+                                        color: white;
+                                    }
+                                    a {
+                                        color: #5375ff;
+                                    }
                                 }
-                            }
                             </style>
                         ''',
-                        f'{len(iommi_sql_debug_log)} queries, {total_duration:.3} seconds total<br><br>',
                     ]
 
+                    for debug_level in SQL_DEBUG_LEVELS:
+                        if debug_level == SQL_DEBUG_LEVEL_OFF:
+                            continue
+                        result.append(f'<a href="?_iommi_sql_trace={debug_level}">{debug_level}</a> ')
+
+                    result.append(f'{len(iommi_sql_debug_log)} queries, {total_duration:.3} seconds total<br><br>')
+
                     if total_duration:
                         color = '#4f4fff'
                         for i, x in enumerate(iommi_sql_debug_log):
@@ -198,7 +209,7 @@ def colorize(text, fg, bold=False):
 
 
 def format_sql(text, width=60, duration=None, short_limit=120):
-    BASE_COLORS = {
+    base_colors = {
         'SELECT': 'green',
         'INSERT': 'magenta',
         'UPDATE': 'orange',
@@ -222,9 +233,9 @@ def format_sql(text, width=60, duration=None, short_limit=120):
             bold = False
             if sql_keyword_re.match(token):
                 bold = True
-                if _first and token in BASE_COLORS:
+                if _first and token in base_colors:
                     _first = False
-                    fg = BASE_COLORS[token]
+                    fg = base_colors[token]
                 if not short:
                     if token in ('AND', 'OR', 'LEFT', 'INNER', 'FROM', 'WHERE', 'ORDER', 'LIMIT'):
                         yield format_html('<span><br>&nbsp;</span>')
@@ -348,10 +359,27 @@ def sql_debug_total_time():
     return sum(x['duration'] for x in log)
 
 
+def get_line_cached_on_request(file_name, line):
+    request = get_current_request()
+
+    if not request:
+        return linecache.getline(file_name, line)
+
+    if not hasattr(request, '_iommi_line_cache'):
+        request._iommi_line_cache = {}
+
+    if file_name not in request._iommi_line_cache:
+        request._iommi_line_cache[file_name] = linecache.getlines(file_name)
+
+    if not request._iommi_line_cache[file_name]:
+        return ''
+    return request._iommi_line_cache[file_name][line - 1]
+
+
 def format_clickable_filename(file_name, line, fn, extra=None):
     if not extra:
         if line is not None:
-            extra = linecache.getline(file_name, line)
+            extra = get_line_cached_on_request(file_name, line)
 
     if extra is None:
         extra = '<unknown>'
@@ -362,15 +390,14 @@ def format_clickable_filename(file_name, line, fn, extra=None):
     if fn is None:
         fn = '<unknown>'
 
-    return f'  File "{file_name}", line {line}, in {fn} => {extra.strip()}'
+    return f'  File "{file_name}", line {line}, in {fn} => {extra.strip()}'.rstrip()
 
 
 def sql_debug_format_stack_trace(frame):
     if frame is None:
         return None
 
-    base_path = os.path.abspath(os.path.join(settings.BASE_DIR, '..')) + "/"
-    msg = []
+    lines = []
     skip_template_code = False
 
     def skip_line(frame):
@@ -380,6 +407,8 @@ def sql_debug_format_stack_trace(frame):
             or 'django/core' in filename
             or 'pydev/pydevd' in filename
             or 'gunicorn' in filename
+            or 'iommi/iommi/declarative/dispatch.py' in filename
+            or 'iommi/iommi/member.py' in filename
         )
 
     while frame:
@@ -388,8 +417,6 @@ def sql_debug_format_stack_trace(frame):
             continue
 
         file_name, line, fn = frame.f_code.co_filename, frame.f_lineno, frame.f_code.co_name
-
-        file_name = file_name.replace(base_path, '')
         extra = ''
         if fn == '_resolve_lookup':
             f = frame
@@ -405,11 +432,11 @@ def sql_debug_format_stack_trace(frame):
         elif skip_template_code:
             skip_template_code = False
 
-        msg.append(format_clickable_filename(file_name, line, fn, extra))
+        lines.append(format_clickable_filename(file_name, line, fn, extra))
 
         frame = frame.f_back
 
-    stack = "\n".join(msg[-20:]).rstrip()
+    stack = "\n".join(lines).rstrip()
     return stack
 
 
@@ -439,7 +466,7 @@ def sql_debug_last_call(response):
                         format_clickable_filename(filename, lineno, str(response.iommi_part._name)), sql_trace=True
                     )
                     sql_debug('With Stack:')
-                sql_debug(logs[0]['stack'], sql_trace=True)
+                sql_debug(logs[-1]['stack'], sql_trace=True)
                 for x in logs[:query_cutoff]:
                     sql_debug_trace_sql(**x)
                 if len(logs) > query_cutoff:
@@ -452,7 +479,7 @@ def sql_debug_last_call(response):
 
     if settings.DEBUG:
         total_sql_time = f" (sql time: {sql_debug_total_time():.3f}s)"
-        duration = f' ({(datetime.now() - request.iommi_start_time).total_seconds():.3f}s)'
+        duration = f' ({(now() - request.iommi_start_time).total_seconds():.3f}s)'
         sql_debug(
             msg=f'{request.META["REQUEST_METHOD"]} {request.get_full_path()} -> {response.status_code} {duration}{total_sql_time}'
         )
@@ -484,6 +511,13 @@ class CursorDebugWrapper(django_db_utils.CursorWrapper):
                 **kwargs,
             )
 
+            self.db.queries_log.append(
+                {
+                    "sql": kwargs['sql'],
+                    "time": "%.3f" % duration,
+                }
+            )
+
     def execute(self, sql, params=None):
         return self._execute_and_log(f=super().execute, sql=sql, params=params)
 
