@@ -26,29 +26,30 @@ class Struct(dict):
 
     def __repr__(self):
         pieces = (
-            "%s=%s" % (key, (repr(val) if val is not self else "%s(...)" % type(self).__name__))
+            "{}={}".format(key, (repr(val) if val is not self else "{}(...)".format(type(self).__name__)))
             for (key, val) in sorted(self.items())
         )
-        return "%s(%s)" % (type(self).__name__, ", ".join(pieces))
+        return "{}({})".format(type(self).__name__, ", ".join(pieces))
 
     __str__ = __repr__
 
     def __getattribute__(self, item):
-        if not dict.__contains__(self, item):
+        try:
+            return dict.__getitem__(self, item)
+        except KeyError:
+            pass
+        try:
+            return object.__getattribute__(self, item)
+        except AttributeError as e:
             try:
-                return object.__getattribute__(self, item)
-            except AttributeError as e:
-                try:
-                    missing_ = object.__getattribute__(self, '__missing__')
-                except AttributeError:
-                    pass
-                else:
-                    return missing_.__get__(self)(item)
-                raise e
-        return dict.__getitem__(self, item)
+                missing_ = object.__getattribute__(self, '__missing__')
+            except AttributeError:
+                pass
+            else:
+                return missing_.__get__(self)(item)
+            raise e
 
-    def __setattr__(self, key, value):
-        self[key] = value
+    __setattr__ = dict.__setitem__
 
     def __delattr__(self, item):
         try:
@@ -60,7 +61,7 @@ class Struct(dict):
         return type(self)(self)
 
 
-class Frozen(object):
+class Frozen:
     """
     Mixin to create an immutable class.
     """
@@ -77,25 +78,25 @@ class Frozen(object):
         return _hash
 
     def __setitem__(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def __setattr__(self, key, value):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def setdefault(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def update(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def clear(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def __delitem__(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def __delattr__(self, *_, **__):
-        raise TypeError("'%s' object attributes are read-only" % (type(self).__name__,))
+        raise TypeError(f"'{type(self).__name__}' object attributes are read-only")
 
     def __reduce__(self):
         return type(self), (), dict(self)
