@@ -1,7 +1,11 @@
+from django.templatetags.static import static
+from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy
 
+from iommi import Fragment
 from iommi.asset import Asset
 from iommi.debug import (
+    endpoint__debug_templates_used,
     endpoint__debug_tree,
     iommi_debug_on,
 )
@@ -10,7 +14,6 @@ from iommi.style import Style
 select2_assets = dict(
     select2_js=Asset.js(attrs__src='https://cdn.jsdelivr.net/npm/select2@4.0.12/dist/js/select2.min.js'),
     select2_css=Asset.css(attrs__href='https://cdn.jsdelivr.net/npm/select2@4.0.12/dist/css/select2.min.css'),
-    select2_init_select2_js=Asset.js(children__text__template='iommi/form/init_select2.js'),
 )
 
 select2_enhanced_forms = Style(
@@ -24,6 +27,9 @@ select2_enhanced_forms = Style(
                 assets=select2_assets,
                 input__attrs__class__select2_enhance=True,
             ),
+            checkboxes=dict(
+                input__attrs__class__select2_enhance=False,
+            ),
             choice_queryset=dict(
                 input__template='iommi/form/choice_select2.html',
                 non_editable_input__attrs__class__select2_enhance=False,
@@ -36,6 +42,19 @@ select2_enhanced_forms = Style(
     ),
 )
 
+iommi_js_init = Fragment(
+    mark_safe(
+        """\
+<script>
+    document.addEventListener("iommi.init.start", (event) => {
+        event.detail.iommi.debug = true;
+    });
+</script>
+"""
+    ),
+    include=lambda **_: iommi_debug_on(),
+)
+
 base = Style(
     internal=True,
     base_template='iommi/base.html',
@@ -45,6 +64,10 @@ base = Style(
             include=lambda endpoint, **_: iommi_debug_on(),
             func=endpoint__debug_tree,
         ),
+        endpoints__debug_templates_used=dict(
+            include=lambda endpoint, **_: iommi_debug_on(),
+            func=endpoint__debug_templates_used,
+        ),
         assets=dict(
             jquery=Asset.js(
                 attrs=dict(
@@ -54,13 +77,27 @@ base = Style(
                 ),
                 after=-1,
             ),
-            axios=Asset.js(
+            iommi_js=Asset.js(
                 attrs=dict(
-                    src='https://cdn.jsdelivr.net/npm/axios@0.21.0/dist/axios.min.js',
-                    integrity='sha256-OPn1YfcEh9W2pwF1iSS+yDk099tYj+plSrCS6Esa9NA=',
-                    crossorigin='anonymous',
+                    src=lambda **_: static('js/iommi.js'),
                 ),
-                after=-1,
+            ),
+            iommi_code_finder_js=Asset.js(
+                attrs=dict(
+                    src=lambda **_: static('js/iommi_code_finder.js'),
+                ),
+                include=lambda request, **_: '_iommi_code_finder' in request.GET if request else False,
+            ),
+            iommi_js_init=iommi_js_init,
+            iommi_scroll_js=Asset.js(
+                attrs=dict(
+                    src=lambda **_: static('js/iommi-scroll.js'),
+                ),
+            ),
+            meta=Asset(
+                mark_safe(
+                    '''<meta content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no" name="viewport">'''
+                )
             ),
         ),
     ),
@@ -86,17 +123,28 @@ base = Style(
             radio=dict(
                 input__template='iommi/form/radio.html',
             ),
+            checkboxes=dict(
+                input__template='iommi/form/checkboxes.html',
+                input__attrs__multiple=None,
+            ),
             heading=dict(
                 template='iommi/form/heading.html',
             ),
         ),
+        non_editable_input=dict(
+            attrs__disabled=lambda fragment, **_: True if fragment.tag in ('input', 'textarea') else None,
+        ),
     ),
     Column=dict(
         shortcuts=dict(
             select=dict(
-                header__attrs__title='Select all',
+                header__attrs__title=gettext_lazy('Select all'),
             ),
-        )
+        ),
+        header__attrs__class__iommi_sort_header=lambda header, **_: header.url is not None,
+    ),
+    HeaderConfig=dict(
+        tag='thead',
     ),
     Paginator=dict(
         show_always=False,
@@ -105,11 +153,16 @@ base = Style(
         container__tag='div',
         active_item__tag='span',
         link__tag='a',
+        link__attrs__class__iommi_page_link=True,
+        attrs={
+            "data-iommi-page-parameter": lambda paginator, **_: paginator.iommi_path,
+            'aria-label': 'Pages',
+        },
     ),
     Query=dict(
         template='iommi/query/form.html',
         advanced__template='iommi/query/advanced.html',
-        assets__ajax_enhance=Asset.js(children__text__template='iommi/query/ajax_enhance.js'),
+        assets__iommi_css=Asset.css(attrs__href=lambda **_: static('css/iommi.css')),
         form__attrs__class__iommi_filter=True,
     ),
     Actions=dict(
@@ -139,6 +192,21 @@ base = Style(
         },
     ),
     LiveEditPage__iommi_style='bootstrap',
+    MainMenu=dict(
+        attrs__class__main_menu=True,
+        assets=dict(
+            iommi_main_menu_css=Asset.css(attrs__href='/static/css/iommi_main_menu.css')
+        ),
+    ),
+    EditColumn=dict(
+        shortcuts=dict(
+            reorder_handle=dict(
+                assets__sortable_js = Asset.js(
+                    attrs__src='https://cdn.jsdelivr.net/npm/sortablejs@latest/Sortable.min.js',
+                )
+            ),
+        ),
+    )
 )
 
 base_enhanced_forms = Style(
