@@ -1,3 +1,5 @@
+from django.utils.translation import gettext_lazy
+
 from iommi.asset import Asset
 from iommi.style import (
     Style,
@@ -55,7 +57,6 @@ bootstrap_base = Style(
         attrs__class={
             'container': True,
             'mt-5': True,
-            'pt-5': True,
         },
     ),
     Field=dict(
@@ -75,6 +76,17 @@ bootstrap_base = Style(
                     'form-control': False,
                 },
             ),
+            checkboxes=dict(
+                attrs__class={
+                    'form-group': False,
+                    'form-check': True,
+                },
+                input__attrs__class={
+                    'form-check-input': True,
+                    'form-control': False,
+                },
+            ),
+            number__input__attrs__class={'text-right': True},
         ),
         attrs__class={
             'form-group': True,
@@ -91,11 +103,14 @@ bootstrap_base = Style(
     FieldGroup=dict(
         tag='div',
         attrs__class={'form-row': True},
-        assets__field_group_select2_css=Asset('''
+        assets__field_group_select2_css=Asset(
+            '''
         .form-group .select2-container {
             display: block;
         }
-        ''', tag='style'),
+        ''',
+            tag='style',
+        ),
     ),
     Action=dict(
         shortcuts=dict(
@@ -123,9 +138,10 @@ bootstrap_base = Style(
         header__attrs__class={'text-nowrap': True},
         shortcuts=dict(
             select=dict(
-                header__attrs__title='Select all',
+                header__attrs__title=gettext_lazy('Select all'),
                 header__attrs__class={'text-center': True},
                 cell__attrs__class={'text-center': True},
+                extra__icon='fa fa-check-square-o',
             ),
             number=dict(
                 cell__attrs__class={'text-right': True},
