@@ -1,3 +1,8 @@
+from textwrap import dedent
+
+from django.utils.safestring import mark_safe
+from django.utils.translation import gettext_lazy
+
 from iommi.asset import Asset
 from iommi.style import (
     Style,
@@ -30,8 +35,8 @@ bootstrap5_base = Style(
     root__assets=dict(
         css=Asset.css(
             attrs=dict(
-                href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css",
-                integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3",
+                href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css",
+                integrity="sha256-PI8n5gCcz9cQqQXm3PEtDuPG8qx9oFsFctPg0S5zb8g=",
                 crossorigin="anonymous",
             ),
         ),
@@ -44,18 +49,41 @@ bootstrap5_base = Style(
         ),
         js=Asset.js(
             attrs=dict(
-                src="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.min.js",
-                integrity="sha384-QJHtvGhmr9XOIpI6YVutG+2QOK9T+ZnN4kzFN1RtK3zEFEIsxhlmWl5/YESvpZ13",
+                src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
+                integrity="sha256-CDOy6cOibCWEdsRiZuaHf8dSGGJRYuBGC+mjoJimHGw=",
                 crossorigin="anonymous",
             )
         ),
+        auto_darkmode=Asset.js(
+            children__source=mark_safe(dedent('''
+                function updateTheme() {
+                    document.querySelector("html").setAttribute("data-bs-theme", window.matchMedia("(prefers-color-scheme: dark)").matches ? "dark" : "light")
+                }
+                window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', updateTheme)
+                updateTheme()             
+            '''))
+        ),
+        select2_dark_mode=Asset(
+            tag='style',
+            children__source=dedent('''
+            [data-bs-theme=dark] .select2-container--default .select2-selection--single,
+            [data-bs-theme=dark] .select2-container--default .select2-results__option,
+            [data-bs-theme=dark] .select2-container--default .select2-selection--multiple {
+                color: white;
+                background: var(--bs-body-bg);;
+            }
+            [data-bs-theme=dark]  .select2-container--default .select2-selection--multiple .select2-selection__choice {
+                color: white;
+                background: #444;
+            }
+            ''')
+        )
     ),
     Container=dict(
         tag='div',
         attrs__class={
             'container': True,
             'mt-5': True,
-            'pt-5': True,
         },
     ),
     Field=dict(
@@ -73,12 +101,14 @@ bootstrap5_base = Style(
                 input__attrs__class={
                     'form-check-input': True,
                     'form-control': False,
+                    'form-select': False,
                 },
             ),
             choice__input__attrs__class={'form-select': True},
         ),
         attrs__class={
             'mb-3': True,
+            'col': lambda field, **_: field.group is not None,
         },
         input__attrs__class={
             'form-control': True,
@@ -121,7 +151,7 @@ bootstrap5_base = Style(
         header__attrs__class={'text-nowrap': True},
         shortcuts=dict(
             select=dict(
-                header__attrs__title='Select all',
+                header__attrs__title=gettext_lazy('Select all'),
                 header__attrs__class={'text-center': True},
                 cell__attrs__class={'text-center': True},
             ),
@@ -174,8 +204,7 @@ bootstrap5_base = Style(
     DebugMenu=dict(
         attrs__class={
             'bg-primary': False,
-            'navbar': False,
-            'navbar-dark': False,
+            'bg-secondary': True,
         },
         items_container__attrs__class={
             'pl-0': True,
