@@ -10,13 +10,8 @@ test = Style(
     font_awesome_4,
     select2_enhanced_forms,
     internal=True,
-    root__assets=dict(
-        jquery=None,
-        select2_js=None,
-        select2_css=None,
-        icons=None,
-        axios=None,
-    ),
+    root__assets=None,
+    root__endpoints=None,
     Field=dict(
         shortcuts=dict(),
     ),
