@@ -1,4 +1,6 @@
 import copy
+import functools
+import inspect
 from typing import (
     Any,
     Dict,
@@ -9,8 +11,9 @@ from iommi.attrs import (
     evaluate_attrs,
 )
 from iommi.base import (
-    items,
     NOT_BOUND_MESSAGE,
+    items,
+    keys,
 )
 from iommi.declarative.namespace import Namespace
 from iommi.evaluate import (
@@ -18,6 +21,7 @@ from iommi.evaluate import (
     evaluate_members,
     evaluate_strict,
     find_static_items,
+    find_static_items_recursively,
     get_callable_description,
     get_signature,
     is_callable,
@@ -25,16 +29,16 @@ from iommi.evaluate import (
     signature_from_kwargs,
 )
 from iommi.refinable import (
-    evaluated_refinable,
     EvaluatedRefinable,
-    get_special_evaluated_attributes,
     Prio,
     Refinable,
-    refinable,
     RefinableMembers,
     RefinableObject,
+    evaluated_refinable,
+    get_special_evaluated_attributes,
+    refinable,
 )
-from iommi.struct import Struct
+from iommi.struct import Struct, merged
 from iommi.style import Style
 
 # Backward compatible definition
@@ -46,6 +50,37 @@ class PathNotFoundException(Exception):
     pass
 
 
+worst_offenders_candidates = set()
+
+
+def trace_worst_offenders(f):
+    @functools.wraps(f)
+    def trace_worst_offenders_inner(*args, **kwargs):
+        from iommi.thread_locals import get_current_request
+
+        request = get_current_request()
+        if request is not None:
+            under_trace = request.GET.get('_iommi_func_worst_offender')
+            if under_trace == f.__name__:
+                import sys
+                from collections import defaultdict
+
+                from iommi.sql_trace import sql_debug_format_stack_trace
+
+                if not hasattr(request, '_iommi_func_worst_offender'):
+                    request._iommi_func_worst_offender = defaultdict(int)
+
+                frame = sys._getframe().f_back.f_back
+                x = sql_debug_format_stack_trace(frame)
+                request._iommi_func_worst_offender[x] += 1
+
+        return f(*args, **kwargs)
+
+    trace_worst_offenders_inner._iommi_can_trace_worst_offenders = True
+    worst_offenders_candidates.add(f.__name__)
+    return trace_worst_offenders_inner
+
+
 class Traversable(RefinableObject):
     """
     Abstract API for objects that have a place in the iommi path structure.
@@ -60,8 +95,6 @@ class Traversable(RefinableObject):
     context = None
 
     iommi_style: str = Refinable()
-    assets = RefinableMembers()
-    endpoints = RefinableMembers()
 
     @staticmethod
     @refinable
@@ -131,6 +164,7 @@ class Traversable(RefinableObject):
         assert self._is_bound, NOT_BOUND_MESSAGE
         return build_long_path(self).replace('/', '__')
 
+    @trace_worst_offenders
     def apply_style(self, iommi_style: Style, is_root=True):
         assert iommi_style.__class__.__name__ == "Style", iommi_style.__class__.__name__
 
@@ -153,9 +187,7 @@ class Traversable(RefinableObject):
 
         attrs = getattr(self, 'attrs', None)
         if attrs:
-            find_static_items(attrs)
-            find_static_items(attrs.get('style', None))
-            find_static_items(attrs.get('class', None))
+            find_static_items_recursively(attrs)
 
         extra_evaluated = getattr(self, 'extra_evaluated', None)
         if extra_evaluated:
@@ -192,13 +224,20 @@ class Traversable(RefinableObject):
             'traversable': result,
         }
         if parent is None:
-            evaluate_parameters['request'] = request
-            if hasattr(request, 'iommi_view_params'):
-                params = request.iommi_view_params
-                extra_params = result.extra_params(request=request, **params)
-                assert isinstance(extra_params, dict), 'extra_params needs to return a dict with additional parameters'
-                params.update(extra_params)
-                evaluate_parameters['params'] = params
+            evaluate_parameters = {
+                'request': request,
+                'user': getattr(request, 'user', None),
+                **evaluate_parameters,
+            }
+
+            params = getattr(request, 'iommi_view_params', Struct())
+
+            extra_params = result.extra_params(request=request, **params)
+            assert isinstance(extra_params, dict), 'extra_params needs to return a dict with additional parameters'
+            params.update(extra_params)
+            evaluate_parameters['params'] = params
+            evaluate_parameters = {**params, **evaluate_parameters}
+
         result._evaluate_parameters = evaluate_parameters
 
         if hasattr(result, 'include'):
@@ -217,7 +256,7 @@ class Traversable(RefinableObject):
         evaluate_members(result, **evaluate_parameters)
 
         if hasattr(result, 'attrs'):
-            result.attrs = evaluate_attrs(result, **result.iommi_evaluate_parameters())
+            result.attrs = evaluate_attrs(result, **evaluate_parameters)
 
         if hasattr(result, 'extra_evaluated'):
             result.extra_evaluated = Struct(evaluate_as_needed(result.extra_evaluated or {}, evaluate_parameters))
@@ -225,7 +264,22 @@ class Traversable(RefinableObject):
         for k in get_special_evaluated_attributes(result):
             v = getattr(result, k)
             if is_callable(v) and not isinstance(v, type):
-                assert False, ('SpecialEvaluatedRefinable not evaluated', k, v, repr(result))
+                arguments = '\n        '.join(keys(result.iommi_evaluate_parameters()))
+                parameters = '\n        '.join(inspect.getfullargspec(v)[0])
+                assert False, f'''SpecialEvaluatedRefinable not evaluated
+
+    Refinable name:
+        {k}
+
+    Path:
+        {result.iommi_dunder_path}
+
+    Possible inputs:
+        {arguments}
+
+    Function inputs:
+        {parameters}
+'''
 
         return result
 
@@ -233,23 +287,25 @@ class Traversable(RefinableObject):
         pass
 
     def own_evaluate_parameters(self):
-        return {}
+        return dict(root=self.iommi_root())
 
     def iommi_evaluate_parameters(self):
         return self._evaluate_parameters
 
     def invoke_callback(self, callback, **kwargs):
+        all_kwargs = merged(self.iommi_evaluate_parameters(), kwargs)
         try:
-            return callback(**kwargs, **self.iommi_evaluate_parameters())
+            return callback(**all_kwargs)
         except TypeError as e:
             if not matches(
-                signature_from_kwargs(dict(**kwargs, **self.iommi_evaluate_parameters())),
+                signature_from_kwargs(all_kwargs),
                 get_signature(callback),
                 __match_empty=True,
             ):
+                keyword_arguments_string = "\n    ".join(sorted([*kwargs, *self.iommi_evaluate_parameters()]))
                 raise TypeError(
                     f'TypeError when invoking callback {get_callable_description(callback)}.\n'
-                    f'(Keyword arguments: {", ".join(sorted([*kwargs, *self.iommi_evaluate_parameters()]))})'
+                    f'Keyword arguments:\n    {keyword_arguments_string}'
                 ) from e
             raise
 
